@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: ohbci.c,v $
                              -------------------
-    cvs         : $Id: ohbci.c,v 1.9 2005/08/24 11:56:24 aquamaniac Exp $
+    cvs         : $Id: ohbci.c,v 1.13 2005/09/29 03:21:55 aquamaniac Exp $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -27,6 +27,7 @@
 #include <gwenhywfar/md.h>
 #include <gwenhywfar/waitcallback.h>
 #include <gwenhywfar/ct_file.h>
+#include <gwenhywfar/text.h> /* DEBUG */
 
 #include <stdlib.h>
 #include <assert.h>
@@ -679,18 +680,18 @@ void GWEN_CryptTokenOHBCI__DecodeKey(GWEN_CRYPTTOKEN *ct,
 
     case GWEN_CRYPTTOKEN_OHBCI_TAG_KEY_P:
       if (p && l)
-        GWEN_DB_SetBinValue(node,
-                            GWEN_DB_FLAGS_OVERWRITE_VARS,
-                            "data/p",
-                            p, l);
+	GWEN_DB_SetBinValue(node,
+			    GWEN_DB_FLAGS_OVERWRITE_VARS,
+			    "data/p",
+			    p, l);
       break;
 
     case GWEN_CRYPTTOKEN_OHBCI_TAG_KEY_Q:
       if (p && l)
-        GWEN_DB_SetBinValue(node,
-                            GWEN_DB_FLAGS_OVERWRITE_VARS,
-                            "data/q",
-                            p, l);
+	GWEN_DB_SetBinValue(node,
+			    GWEN_DB_FLAGS_OVERWRITE_VARS,
+			    "data/q",
+			    p, l);
       break;
 
     case GWEN_CRYPTTOKEN_OHBCI_TAG_KEY_D:
@@ -703,26 +704,26 @@ void GWEN_CryptTokenOHBCI__DecodeKey(GWEN_CRYPTTOKEN *ct,
 
     case GWEN_CRYPTTOKEN_OHBCI_TAG_KEY_DMP1:
       if (p && l)
-        GWEN_DB_SetBinValue(node,
-                            GWEN_DB_FLAGS_OVERWRITE_VARS,
-                            "data/dmp1",
-                            p, l);
+	GWEN_DB_SetBinValue(node,
+			    GWEN_DB_FLAGS_OVERWRITE_VARS,
+			    "data/dmp1",
+			    p, l);
       break;
 
     case GWEN_CRYPTTOKEN_OHBCI_TAG_KEY_DMQ1:
       if (p && l)
-        GWEN_DB_SetBinValue(node,
-                            GWEN_DB_FLAGS_OVERWRITE_VARS,
-                            "data/dmq1",
-                            p, l);
+	GWEN_DB_SetBinValue(node,
+			    GWEN_DB_FLAGS_OVERWRITE_VARS,
+			    "data/dmq1",
+			    p, l);
       break;
 
     case GWEN_CRYPTTOKEN_OHBCI_TAG_KEY_IQMP:
       if (p && l)
-        GWEN_DB_SetBinValue(node,
-                            GWEN_DB_FLAGS_OVERWRITE_VARS,
-                            "data/iqmp",
-                            p, l);
+	GWEN_DB_SetBinValue(node,
+			    GWEN_DB_FLAGS_OVERWRITE_VARS,
+			    "data/iqmp",
+			    p, l);
       break;
 
     default:
@@ -1870,5 +1871,11 @@ int GWEN_CryptTokenOHBCI_Close(GWEN_CRYPTTOKEN *ct){
     return rv;
   }
 
+  memset(lct->password, 0, sizeof(lct->password));
+  lct->passWordIsSet=0;
+
   return 0;
 }
+
+
+
