@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: list.c,v $
                              -------------------
-    cvs         : $Id: list.c,v 1.20 2005/08/04 04:55:45 aquamaniac Exp $
+    cvs         : $Id: list.c,v 1.21 2005/09/28 19:10:11 aquamaniac Exp $
     begin       : Sat Nov 15 2003
     copyright   : (C) 2003 by Martin Preuss
     email       : martin@libchipcard.de
@@ -516,6 +516,27 @@ void GWEN_List_Erase(GWEN_LIST *l, GWEN_LIST_ITERATOR *it){
 
 
 
+void GWEN_List_Remove(GWEN_LIST *l, const void *p) {
+  GWEN_LIST_ITERATOR *li;
+
+  li=GWEN_List_First(l);
+  if (li) {
+    void *d;
+
+    d=GWEN_ListIterator_Data(li);
+    while(d) {
+      if (d==p) {
+        GWEN_List_Erase(l, li);
+        break;
+      }
+      d=GWEN_ListIterator_Next(li);
+    }
+    GWEN_ListIterator_free(li);
+  }
+}
+
+
+
 GWEN_LIST_ITERATOR *GWEN_List_First(const GWEN_LIST *l){
   GWEN_LIST_ITERATOR *li;
 
