@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: crypt.c,v $
  -------------------
- cvs         : $Id: crypt.c,v 1.31 2005/08/04 04:55:45 aquamaniac Exp $
+ cvs         : $Id: crypt.c,v 1.35 2005/09/29 03:21:55 aquamaniac Exp $
  begin       : Thu Nov 06 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de
@@ -59,6 +59,7 @@ static char gwen_random_state[GWEN_RANDSTATE_BUFSIZE];
 
 
 GWEN_INHERIT_FUNCTIONS(GWEN_CRYPTKEY)
+GWEN_LIST_FUNCTIONS(GWEN_CRYPTKEY, GWEN_CryptKey)
 GWEN_LIST2_FUNCTIONS(GWEN_CRYPTKEY, GWEN_CryptKey)
 
 
@@ -176,21 +177,30 @@ GWEN_CRYPTKEY *GWEN_CryptKey_new(){
 
   GWEN_NEW_OBJECT(GWEN_CRYPTKEY, ck);
   GWEN_INHERIT_INIT(GWEN_CRYPTKEY, ck);
+  GWEN_LIST_INIT(GWEN_CRYPTKEY, ck);
   DBG_MEM_INC("GWEN_CRYPTKEY", 0);
 
   ck->keyspec=GWEN_KeySpec_new();
+  ck->usage=1;
   return ck;
 }
 
 
+
 void GWEN_CryptKey_free(GWEN_CRYPTKEY *key){
   if (key) {
-    DBG_MEM_DEC("GWEN_CRYPTKEY");
-    GWEN_INHERIT_FINI(GWEN_CRYPTKEY, key);
-    if (key->freeKeyDataFn)
-      key->freeKeyDataFn(key);
-    GWEN_KeySpec_free(key->keyspec);
-    free(key);
+    assert(key->usage);
+    if (key->usage==1) {
+      DBG_MEM_DEC("GWEN_CRYPTKEY");
+      GWEN_LIST_FINI(GWEN_CRYPTKEY, key);
+      GWEN_INHERIT_FINI(GWEN_CRYPTKEY, key);
+      if (key->freeKeyDataFn)
+        key->freeKeyDataFn(key);
+      GWEN_KeySpec_free(key->keyspec);
+      GWEN_FREE_OBJECT(key);
+    }
+    else
+      key->usage--;
   }
 }
 
@@ -200,6 +210,7 @@ GWEN_CRYPTKEY *GWEN_CryptKey_dup(const GWEN_CRYPTKEY *key){
   GWEN_CRYPTKEY *newKey;
 
   assert(key);
+  assert(key->usage);
   assert(key->dupFn);
   newKey=key->dupFn(key);
   assert(key->keyspec);
@@ -228,6 +239,7 @@ GWEN_ERRORCODE GWEN_CryptKey_Encrypt(const GWEN_CRYPTKEY *key,
                                      GWEN_BUFFER *src,
                                      GWEN_BUFFER *dst){
   assert(key);
+  assert(key->usage);
   assert(src);
   assert(dst);
   assert(key->encryptFn);
@@ -240,6 +252,7 @@ GWEN_ERRORCODE GWEN_CryptKey_Decrypt(const GWEN_CRYPTKEY *key,
                                      GWEN_BUFFER *src,
                                      GWEN_BUFFER *dst){
   assert(key);
+  assert(key->usage);
   assert(src);
   assert(dst);
   assert(key->decryptFn);
@@ -252,6 +265,7 @@ GWEN_ERRORCODE GWEN_CryptKey_Sign(const GWEN_CRYPTKEY *key,
                                   GWEN_BUFFER *src,
                                   GWEN_BUFFER *dst){
   assert(key);
+  assert(key->usage);
   assert(src);
   assert(dst);
   assert(key->signFn);
@@ -264,6 +278,7 @@ GWEN_ERRORCODE GWEN_CryptKey_Verify(const GWEN_CRYPTKEY *key,
                                     GWEN_BUFFER *src,
                                     GWEN_BUFFER *dst){
   assert(key);
+  assert(key->usage);
   assert(src);
   assert(dst);
   assert(key->verifyFn);
@@ -274,8 +289,10 @@ GWEN_ERRORCODE GWEN_CryptKey_Verify(const GWEN_CRYPTKEY *key,
 
 unsigned int GWEN_CryptKey_GetChunkSize(const GWEN_CRYPTKEY *key){
   assert(key);
-  assert(key->getChunkSizeFn);
-  return key->getChunkSizeFn(key);
+  assert(key->usage);
+  if (key->getChunkSizeFn)
+    return key->getChunkSizeFn(key);
+  return 0;
 }
 
 
@@ -319,6 +336,7 @@ GWEN_ERRORCODE GWEN_CryptKey_ToDb(const GWEN_CRYPTKEY *key,
   GWEN_DB_NODE *gr;
 
   assert(key);
+  assert(key->usage);
   if (GWEN_KeySpec_ToDb(key->keyspec, db)) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Could not store keyspec in DB");
     return GWEN_Error_new(0,
@@ -328,6 +346,7 @@ GWEN_ERRORCODE GWEN_CryptKey_ToDb(const GWEN_CRYPTKEY *key,
   }
   GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                       "flags", key->flags);
+
   /* save key specific data */
   gr=GWEN_DB_GetGroup(db,
                       GWEN_DB_FLAGS_DEFAULT |
@@ -346,6 +365,7 @@ GWEN_ERRORCODE GWEN_CryptKey_SetData(GWEN_CRYPTKEY *key,
   GWEN_ERRORCODE err;
 
   assert(key);
+  assert(key->usage);
   assert(buffer);
   assert(bsize);
   n=GWEN_DB_Group_new("data");
@@ -376,6 +396,7 @@ GWEN_ERRORCODE GWEN_CryptKey_GetData(GWEN_CRYPTKEY *key,
   unsigned int size;
 
   assert(key);
+  assert(key->usage);
   assert(buffer);
   assert(*bsize);
   assert(key->toDbFn);
@@ -414,10 +435,14 @@ GWEN_ERRORCODE GWEN_CryptKey_GetData(GWEN_CRYPTKEY *key,
 
 
 GWEN_ERRORCODE GWEN_CryptKey_Generate(GWEN_CRYPTKEY *key,
-                                      unsigned keylength){
+                                      unsigned int keylength){
+  GWEN_ERRORCODE err;
+
   assert(key);
+  assert(key->usage);
   assert(key->generateKeyFn);
-  return key->generateKeyFn(key, keylength);
+  err=key->generateKeyFn(key, keylength);
+  return err;
 }
 
 
@@ -454,6 +479,7 @@ int GWEN_CryptKey_FromPassword(const char *password,
 
 GWEN_ERRORCODE GWEN_CryptKey_Open(GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   assert(key->openFn);
   return key->openFn(key);
 }
@@ -462,6 +488,7 @@ GWEN_ERRORCODE GWEN_CryptKey_Open(GWEN_CRYPTKEY *key){
 
 GWEN_ERRORCODE GWEN_CryptKey_Close(GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   assert(key->closeFn);
   return key->closeFn(key);
 }
@@ -470,6 +497,7 @@ GWEN_ERRORCODE GWEN_CryptKey_Close(GWEN_CRYPTKEY *key){
 
 int GWEN_CryptKey_GetStatus(const GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   return GWEN_KeySpec_GetStatus(key->keyspec);
 }
 
@@ -477,6 +505,7 @@ int GWEN_CryptKey_GetStatus(const GWEN_CRYPTKEY *key){
 
 void GWEN_CryptKey_SetStatus(GWEN_CRYPTKEY *key, int i){
   assert(key);
+  assert(key->usage);
   GWEN_KeySpec_SetStatus(key->keyspec, i);
 }
 
@@ -484,6 +513,7 @@ void GWEN_CryptKey_SetStatus(GWEN_CRYPTKEY *key, int i){
 
 const char *GWEN_CryptKey_GetKeyType(const GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   return GWEN_KeySpec_GetKeyType(key->keyspec);
 }
 
@@ -492,6 +522,7 @@ const char *GWEN_CryptKey_GetKeyType(const GWEN_CRYPTKEY *key){
 void GWEN_CryptKey_SetKeyType(GWEN_CRYPTKEY *key,
                               const char *s){
   assert(key);
+  assert(key->usage);
   assert(s);
   GWEN_KeySpec_SetKeyType(key->keyspec, s);
 }
@@ -500,6 +531,7 @@ void GWEN_CryptKey_SetKeyType(GWEN_CRYPTKEY *key,
 
 const char *GWEN_CryptKey_GetKeyName(const GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   return GWEN_KeySpec_GetKeyName(key->keyspec);
 }
 
@@ -508,6 +540,7 @@ const char *GWEN_CryptKey_GetKeyName(const GWEN_CRYPTKEY *key){
 void GWEN_CryptKey_SetKeyName(GWEN_CRYPTKEY *key,
                               const char *s){
   assert(key);
+  assert(key->usage);
   assert(s);
   GWEN_KeySpec_SetKeyName(key->keyspec, s);
 }
@@ -516,6 +549,7 @@ void GWEN_CryptKey_SetKeyName(GWEN_CRYPTKEY *key,
 
 const char *GWEN_CryptKey_GetOwner(const GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   assert(key->keyspec);
   return GWEN_KeySpec_GetOwner(key->keyspec);
 }
@@ -525,6 +559,7 @@ const char *GWEN_CryptKey_GetOwner(const GWEN_CRYPTKEY *key){
 void GWEN_CryptKey_SetOwner(GWEN_CRYPTKEY *key,
                             const char *s){
   assert(key);
+  assert(key->usage);
   assert(key->keyspec);
   GWEN_KeySpec_SetOwner(key->keyspec, s);
 }
@@ -533,6 +568,7 @@ void GWEN_CryptKey_SetOwner(GWEN_CRYPTKEY *key,
 
 unsigned int GWEN_CryptKey_GetNumber(const GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   assert(key->keyspec);
   return GWEN_KeySpec_GetNumber(key->keyspec);
 }
@@ -542,6 +578,7 @@ unsigned int GWEN_CryptKey_GetNumber(const GWEN_CRYPTKEY *key){
 void GWEN_CryptKey_SetNumber(GWEN_CRYPTKEY *key,
                              unsigned int i){
   assert(key);
+  assert(key->usage);
   assert(key->keyspec);
   GWEN_KeySpec_SetNumber(key->keyspec, i);
 }
@@ -550,6 +587,7 @@ void GWEN_CryptKey_SetNumber(GWEN_CRYPTKEY *key,
 
 unsigned int GWEN_CryptKey_GetVersion(const GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   assert(key->keyspec);
   return GWEN_KeySpec_GetVersion(key->keyspec);
 }
@@ -559,6 +597,7 @@ unsigned int GWEN_CryptKey_GetVersion(const GWEN_CRYPTKEY *key){
 void GWEN_CryptKey_SetVersion(GWEN_CRYPTKEY *key,
                               unsigned int i){
   assert(key);
+  assert(key->usage);
   assert(key->keyspec);
   GWEN_KeySpec_SetVersion(key->keyspec, i);
 }
@@ -567,6 +606,7 @@ void GWEN_CryptKey_SetVersion(GWEN_CRYPTKEY *key,
 
 void *GWEN_CryptKey_GetKeyData(const GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   return key->keyData;
 }
 
@@ -575,6 +615,7 @@ void *GWEN_CryptKey_GetKeyData(const GWEN_CRYPTKEY *key){
 void GWEN_CryptKey_SetKeyData(GWEN_CRYPTKEY *key,
                               void *kd){
   assert(key);
+  assert(key->usage);
   assert(kd);
 
   if (key->keyData && key->freeKeyDataFn)
@@ -586,6 +627,7 @@ void GWEN_CryptKey_SetKeyData(GWEN_CRYPTKEY *key,
 
 const GWEN_KEYSPEC *GWEN_CryptKey_GetKeySpec(const GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   return key->keyspec;
 }
 
@@ -594,6 +636,7 @@ const GWEN_KEYSPEC *GWEN_CryptKey_GetKeySpec(const GWEN_CRYPTKEY *key){
 void GWEN_CryptKey_SetKeySpec(GWEN_CRYPTKEY *key,
                               const GWEN_KEYSPEC *cks){
   assert(key);
+  assert(key->usage);
   assert(key->keyspec);
   assert(cks);
   GWEN_KeySpec_free(key->keyspec);
@@ -604,6 +647,7 @@ void GWEN_CryptKey_SetKeySpec(GWEN_CRYPTKEY *key,
 
 int GWEN_CryptKey_GetOpenCount(const GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   return key->openCount;
 }
 
@@ -611,6 +655,7 @@ int GWEN_CryptKey_GetOpenCount(const GWEN_CRYPTKEY *key){
 
 void GWEN_CryptKey_IncrementOpenCount(GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   key->openCount++;
 }
 
@@ -618,6 +663,7 @@ void GWEN_CryptKey_IncrementOpenCount(GWEN_CRYPTKEY *key){
 
 void GWEN_CryptKey_DecrementOpenCount(GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   if (key->openCount>0)
     key->openCount--;
   else {
@@ -629,6 +675,7 @@ void GWEN_CryptKey_DecrementOpenCount(GWEN_CRYPTKEY *key){
 void GWEN_CryptKey_SetEncryptFn(GWEN_CRYPTKEY *key,
                                 GWEN_CRYPTKEY_ENCRYPT_FN encryptFn){
   assert(key);
+  assert(key->usage);
   key->encryptFn=encryptFn;
 }
 
@@ -637,6 +684,7 @@ void GWEN_CryptKey_SetEncryptFn(GWEN_CRYPTKEY *key,
 void GWEN_CryptKey_SetDecryptFn(GWEN_CRYPTKEY *key,
                                 GWEN_CRYPTKEY_DECRYPT_FN decryptFn){
   assert(key);
+  assert(key->usage);
   key->decryptFn=decryptFn;
 }
 
@@ -645,6 +693,7 @@ void GWEN_CryptKey_SetDecryptFn(GWEN_CRYPTKEY *key,
 void GWEN_CryptKey_SetSignFn(GWEN_CRYPTKEY *key,
                              GWEN_CRYPTKEY_SIGN_FN signFn){
   assert(key);
+  assert(key->usage);
   key->signFn=signFn;
 }
 
@@ -653,6 +702,7 @@ void GWEN_CryptKey_SetSignFn(GWEN_CRYPTKEY *key,
 void GWEN_CryptKey_SetVerifyFn(GWEN_CRYPTKEY *key,
                                GWEN_CRYPTKEY_VERIFY_FN verifyFn){
   assert(key);
+  assert(key->usage);
   key->verifyFn=verifyFn;
 }
 
@@ -661,6 +711,7 @@ void GWEN_CryptKey_SetVerifyFn(GWEN_CRYPTKEY *key,
 void GWEN_CryptKey_SetGetChunkSizeFn(GWEN_CRYPTKEY *key,
                                      GWEN_CRYPTKEY_GETCHUNKSIZE_FN getChunkSizeFn){
   assert(key);
+  assert(key->usage);
   key->getChunkSizeFn=getChunkSizeFn;
 }
 
@@ -669,6 +720,7 @@ void GWEN_CryptKey_SetGetChunkSizeFn(GWEN_CRYPTKEY *key,
 void GWEN_CryptKey_SetFromDbFn(GWEN_CRYPTKEY *key,
                                GWEN_CRYPTKEY_FROMDB_FN fromDbFn){
   assert(key);
+  assert(key->usage);
   key->fromDbFn=fromDbFn;
 }
 
@@ -677,6 +729,7 @@ void GWEN_CryptKey_SetFromDbFn(GWEN_CRYPTKEY *key,
 void GWEN_CryptKey_SetToDbFn(GWEN_CRYPTKEY *key,
                              GWEN_CRYPTKEY_TODB_FN toDbFn){
   assert(key);
+  assert(key->usage);
   key->toDbFn=toDbFn;
 }
 
@@ -685,6 +738,7 @@ void GWEN_CryptKey_SetToDbFn(GWEN_CRYPTKEY *key,
 void GWEN_CryptKey_SetGenerateKeyFn(GWEN_CRYPTKEY *key,
                                     GWEN_CRYPTKEY_GENERATEKEY_FN generateKeyFn){
   assert(key);
+  assert(key->usage);
   key->generateKeyFn=generateKeyFn;
 }
 
@@ -693,6 +747,7 @@ void GWEN_CryptKey_SetGenerateKeyFn(GWEN_CRYPTKEY *key,
 void GWEN_CryptKey_SetFreeKeyDataFn(GWEN_CRYPTKEY *key,
                                     GWEN_CRYPTKEY_FREEKEYDATA_FN freeKeyDataFn){
   assert(key);
+  assert(key->usage);
   key->freeKeyDataFn=freeKeyDataFn;
 }
 
@@ -701,6 +756,7 @@ void GWEN_CryptKey_SetFreeKeyDataFn(GWEN_CRYPTKEY *key,
 void GWEN_CryptKey_SetOpenFn(GWEN_CRYPTKEY *key,
                              GWEN_CRYPTKEY_OPEN_FN openFn){
   assert(key);
+  assert(key->usage);
   key->openFn=openFn;
 }
 
@@ -709,6 +765,7 @@ void GWEN_CryptKey_SetOpenFn(GWEN_CRYPTKEY *key,
 void GWEN_CryptKey_SetCloseFn(GWEN_CRYPTKEY *key,
                               GWEN_CRYPTKEY_CLOSE_FN closeKeyFn){
   assert(key);
+  assert(key->usage);
   key->closeFn=closeKeyFn;
 }
 
@@ -717,6 +774,7 @@ void GWEN_CryptKey_SetCloseFn(GWEN_CRYPTKEY *key,
 void GWEN_CryptKey_SetDupFn(GWEN_CRYPTKEY *key,
                             GWEN_CRYPTKEY_DUP_FN dupFn){
   assert(key);
+  assert(key->usage);
   key->dupFn=dupFn;
 }
 
@@ -724,6 +782,7 @@ void GWEN_CryptKey_SetDupFn(GWEN_CRYPTKEY *key,
 
 int GWEN_CryptKey_IsPublic(const GWEN_CRYPTKEY *key){
   assert(key);
+  assert(key->usage);
   return key->pub;
 }
 
@@ -731,11 +790,61 @@ int GWEN_CryptKey_IsPublic(const GWEN_CRYPTKEY *key){
 
 void GWEN_CryptKey_SetPublic(GWEN_CRYPTKEY *key, int i){
   assert(key);
+  assert(key->usage);
   key->pub=i;
 }
 
 
 
+GWEN_TYPE_UINT32 GWEN_CryptKey_GetFlags(const GWEN_CRYPTKEY *key){
+  assert(key);
+  assert(key->usage);
+  return key->flags;
+}
+
+
+
+void GWEN_CryptKey_SetFlags(GWEN_CRYPTKEY *key, GWEN_TYPE_UINT32 fl){
+  assert(key);
+  assert(key->usage);
+  key->flags=fl;
+}
+
+
+
+void GWEN_CryptKey_AddFlags(GWEN_CRYPTKEY *key, GWEN_TYPE_UINT32 fl){
+  assert(key);
+  assert(key->usage);
+  key->flags|=fl;
+}
+
+
+
+void GWEN_CryptKey_SubFlags(GWEN_CRYPTKEY *key, GWEN_TYPE_UINT32 fl){
+  assert(key);
+  assert(key->usage);
+  key->flags&=~fl;
+}
+
+
+long int GWEN_Random(){
+  long int result;
+#ifdef HAVE_RANDOM
+  char* prev_randstate = setstate(gwen_random_state);
+  result = random();
+  setstate(prev_randstate);
+#else
+  result = rand();
+#endif
+  return result;
+}
+
+
+
+
+
+
+
 
 
 GWEN_CRYPTKEY_PROVIDER *GWEN_CryptProvider_new(){
@@ -858,7 +967,7 @@ void GWEN_Crypt_UnregisterAllProviders(){
 
 GWEN_CRYPTKEY *GWEN_CryptKey_List2__freeAll_cb(GWEN_CRYPTKEY *st, void *user_data) {
   GWEN_CryptKey_free(st);
-return 0;
+  return 0;
 }
 
 
@@ -871,42 +980,3 @@ void GWEN_CryptKey_List2_freeAll(GWEN_CRYPTKEY_LIST2 *stl) {
 
 
 
-GWEN_TYPE_UINT32 GWEN_CryptKey_GetFlags(const GWEN_CRYPTKEY *key){
-  assert(key);
-  return key->flags;
-}
-
-
-
-void GWEN_CryptKey_SetFlags(GWEN_CRYPTKEY *key, GWEN_TYPE_UINT32 fl){
-  assert(key);
-  key->flags=fl;
-}
-
-
-
-void GWEN_CryptKey_AddFlags(GWEN_CRYPTKEY *key, GWEN_TYPE_UINT32 fl){
-  assert(key);
-  key->flags|=fl;
-}
-
-
-
-void GWEN_CryptKey_SubFlags(GWEN_CRYPTKEY *key, GWEN_TYPE_UINT32 fl){
-  assert(key);
-  key->flags&=~fl;
-}
-
-
-long int GWEN_Random()
-{
-  long int result;
-#ifdef HAVE_RANDOM
-  char* prev_randstate = setstate(gwen_random_state);
-  result = random();
-  setstate(prev_randstate);
-#else
-  result = rand();
-#endif
-  return result;
-}
