@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: cryptssl_rsa.c,v $
  -------------------
- cvs         : $Id: cryptssl_rsa.c,v 1.21 2005/08/04 04:55:45 aquamaniac Exp $
+ cvs         : $Id: cryptssl_rsa.c,v 1.25 2005/09/29 03:21:55 aquamaniac Exp $
  begin       : Thu Nov 06 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de
@@ -42,6 +42,7 @@
 
 #include <openssl/rsa.h>
 #include <openssl/objects.h>
+#include <openssl/err.h>
 
 #include "cryptssl_p.h"
 
@@ -56,11 +57,36 @@ void GWEN_CryptKeyRSA_FreeKeyData(GWEN_CRYPTKEY *key){
 
 
 
+void GWEN_CryptKeyRSA_DumpPubKey(const GWEN_CRYPTKEY *key){
+  GWEN_DB_NODE *dbDebug;
+  GWEN_ERRORCODE lerr;
+
+  dbDebug=GWEN_DB_Group_new("key");
+  lerr=GWEN_CryptKey_ToDb(key, dbDebug, 1);
+  if (!GWEN_Error_IsOk(lerr)) {
+    DBG_ERROR_ERR(GWEN_LOGDOMAIN, lerr);
+  }
+  else {
+    const void *p;
+    unsigned int len;
+
+    p=GWEN_DB_GetBinValue(dbDebug, "data/n", 0, 0, 0, &len);
+    DBG_ERROR(GWEN_LOGDOMAIN,
+              "Key data follows (%d bytes):", len);
+    GWEN_DB_Dump(dbDebug, stderr, 2);
+    if (p && len)
+      GWEN_Text_DumpString((const char*)p, len, stderr, 2);
+  }
+  GWEN_DB_Group_free(dbDebug);
+}
+
+
+
 GWEN_ERRORCODE GWEN_CryptKeyRSA_Encrypt(const GWEN_CRYPTKEY *key,
                                         GWEN_BUFFER *src,
                                         GWEN_BUFFER *dst){
   unsigned int srclen;
-  unsigned int dstlen;
+  int dstlen;
   unsigned char *psrc;
   unsigned char *pdst;
 
@@ -72,6 +98,12 @@ GWEN_ERRORCODE GWEN_CryptKeyRSA_Encrypt(const GWEN_CRYPTKEY *key,
   if (srclen!=GWEN_CryptKey_GetChunkSize(key)) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Size %d!=%d",
               srclen, GWEN_CryptKey_GetChunkSize(key));
+    if (1) {
+      DBG_ERROR(GWEN_LOGDOMAIN,
+                "Offending key follows");
+      GWEN_CryptKeyRSA_DumpPubKey(key);
+    }
+
     return GWEN_Error_new(0,
                           GWEN_ERROR_SEVERITY_ERR,
                           GWEN_Error_FindType(GWEN_CRYPT_ERROR_TYPE),
@@ -89,7 +121,32 @@ GWEN_ERRORCODE GWEN_CryptKeyRSA_Encrypt(const GWEN_CRYPTKEY *key,
   dstlen=RSA_public_encrypt(srclen, psrc, pdst,
                             GWEN_CryptKey_GetKeyData(key),
                             RSA_NO_PADDING);
-  if (dstlen!=srclen) {
+  if (dstlen==-1) {
+    char errbuf[256];
+
+    ERR_error_string_n(ERR_get_error(), errbuf, sizeof(errbuf));
+    DBG_ERROR(GWEN_LOGDOMAIN,
+              "OpenSSL error: %s", errbuf);
+    ERR_print_errors_fp(stderr);
+    if (1) {
+      DBG_ERROR(GWEN_LOGDOMAIN,
+                "Offending key follows");
+      GWEN_CryptKeyRSA_DumpPubKey(key);
+    }
+    return GWEN_Error_new(0,
+                          GWEN_ERROR_SEVERITY_ERR,
+                          GWEN_Error_FindType(GWEN_CRYPT_ERROR_TYPE),
+                          GWEN_CRYPT_ERROR_ENCRYPT);
+  }
+  if ((unsigned int)dstlen!=srclen) {
+    DBG_ERROR(GWEN_LOGDOMAIN,
+              "ERROR: Unexpected dstlen (%d != %u)\n",
+              dstlen, srclen);
+    if (1) {
+      DBG_ERROR(GWEN_LOGDOMAIN,
+                "Offending key follows");
+      GWEN_CryptKeyRSA_DumpPubKey(key);
+    }
     return GWEN_Error_new(0,
                           GWEN_ERROR_SEVERITY_ERR,
                           GWEN_Error_FindType(GWEN_CRYPT_ERROR_TYPE),
@@ -183,9 +240,7 @@ GWEN_ERRORCODE GWEN_CryptKeyRSA_SignBigNum(const GWEN_CRYPTKEY *key,
   BN_CTX_start(bnctx);
   res=BN_mod_exp(bnresult, bnhash, kd->d, kd->n, bnctx);
 
-  /* FIXME: The description is quite the opposite of the code, while the
-   * code seems to be working (at least when verifying my own signatures)
-   * the iso9796-appendix is as follows:
+  /* the iso9796-appendix is as follows:
    * if (the calculated signature - the modulus) < (the calculated signature)
    * use (the calculated signature - the modulus) as signature
    */
@@ -393,11 +448,13 @@ GWEN_ERRORCODE GWEN_CryptKeyRSA_Verify(const GWEN_CRYPTKEY *key,
 
 unsigned int GWEN_CryptKeyRSA_GetChunkSize(const GWEN_CRYPTKEY *key){
   RSA *kd;
+  unsigned int i;
 
   assert(key);
   kd=(RSA*)GWEN_CryptKey_GetKeyData(key);
   assert(kd);
-  return RSA_size(kd);
+  i=RSA_size(kd);
+  return i;
 }
 
 
@@ -491,6 +548,7 @@ GWEN_ERRORCODE GWEN_CryptKeyRSA_FromDb(GWEN_CRYPTKEY *key,
   }
 
   GWEN_CryptKey_SetKeyData(key, kd);
+
   return 0;
 }
 
@@ -580,7 +638,7 @@ GWEN_ERRORCODE GWEN_CryptKeyRSA_ToDb(const GWEN_CRYPTKEY *key,
 
 
 GWEN_ERRORCODE GWEN_CryptKeyRSA_Generate(GWEN_CRYPTKEY *key,
-                                         unsigned keylength){
+                                         unsigned int keylength){
   RSA *newKey;
 
   assert(key);
@@ -591,7 +649,6 @@ GWEN_ERRORCODE GWEN_CryptKeyRSA_Generate(GWEN_CRYPTKEY *key,
                           NULL, NULL);
   assert(newKey);
   GWEN_CryptKey_SetKeyData(key, newKey);
-
   return 0;
 }
 
