@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: crypttoken.c,v $
  -------------------
- cvs         : $Id: crypttoken.c,v 1.13 2005/08/24 13:56:50 aquamaniac Exp $
+ cvs         : $Id: crypttoken.c,v 1.16 2005/09/09 15:02:28 aquamaniac Exp $
  begin       : Thu Sep 11 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de
@@ -1488,7 +1488,7 @@ GWEN_CRYPTTOKEN *GWEN_CryptToken_new(GWEN_PLUGIN_MANAGER *pm,
   GWEN_NEW_OBJECT(GWEN_CRYPTTOKEN, ct);
   GWEN_LIST_INIT(GWEN_CRYPTTOKEN, ct);
   GWEN_INHERIT_INIT(GWEN_CRYPTTOKEN, ct);
-
+  ct->usage=1;
   ct->cryptManager=pm;
   ct->deviceType=devType;
   if (name)
@@ -1510,18 +1510,23 @@ GWEN_CRYPTTOKEN *GWEN_CryptToken_new(GWEN_PLUGIN_MANAGER *pm,
 
 void GWEN_CryptToken_free(GWEN_CRYPTTOKEN *ct){
   if (ct) {
-    GWEN_INHERIT_FINI(GWEN_CRYPTTOKEN, ct);
-    GWEN_LIST_FINI(GWEN_CRYPTTOKEN, ct);
-    free(ct->tokenName);
-    free(ct->tokenSubType);
-    free(ct->tokenType);
-    free(ct->descriptiveName);
-    GWEN_CryptToken_User_List_free(ct->userList);
-    GWEN_CryptToken_Context_List_free(ct->contextList);
-    GWEN_CryptToken_SignInfo_List_free(ct->signInfoList);
-    GWEN_CryptToken_CryptInfo_List_free(ct->cryptInfoList);
-    GWEN_CryptToken_KeyInfo_List_free(ct->keyInfoList);
-    GWEN_FREE_OBJECT(ct);
+    assert(ct->usage);
+    if (ct->usage==1) {
+      GWEN_INHERIT_FINI(GWEN_CRYPTTOKEN, ct);
+      GWEN_LIST_FINI(GWEN_CRYPTTOKEN, ct);
+      free(ct->tokenName);
+      free(ct->tokenSubType);
+      free(ct->tokenType);
+      free(ct->descriptiveName);
+      GWEN_CryptToken_User_List_free(ct->userList);
+      GWEN_CryptToken_Context_List_free(ct->contextList);
+      GWEN_CryptToken_SignInfo_List_free(ct->signInfoList);
+      GWEN_CryptToken_CryptInfo_List_free(ct->cryptInfoList);
+      GWEN_CryptToken_KeyInfo_List_free(ct->keyInfoList);
+      GWEN_FREE_OBJECT(ct);
+    }
+    else
+      ct->usage--;
   }
 }
 
@@ -1618,6 +1623,7 @@ int GWEN_CryptToken_ReadXml(GWEN_CRYPTTOKEN *ct, GWEN_XMLNODE *n) {
   const char *s;
 
   assert(ct);
+  assert(ct->usage);
   assert(n);
 
   s=GWEN_XMLNode_GetProperty(n, "subTypeName", 0);
@@ -1731,6 +1737,7 @@ int GWEN_CryptToken_ReadXml(GWEN_CRYPTTOKEN *ct, GWEN_XMLNODE *n) {
 GWEN_PLUGIN_MANAGER*
 GWEN_CryptToken_GetCryptManager(const GWEN_CRYPTTOKEN *ct){
   assert(ct);
+  assert(ct->usage);
   return ct->cryptManager;
 }
 
@@ -1738,6 +1745,7 @@ GWEN_CryptToken_GetCryptManager(const GWEN_CRYPTTOKEN *ct){
 
 const char *GWEN_CryptToken_GetTokenName(const GWEN_CRYPTTOKEN *ct){
   assert(ct);
+  assert(ct->usage);
   return ct->tokenName;
 }
 
@@ -1745,6 +1753,7 @@ const char *GWEN_CryptToken_GetTokenName(const GWEN_CRYPTTOKEN *ct){
 
 void GWEN_CryptToken_SetTokenName(GWEN_CRYPTTOKEN *ct, const char *s){
   assert(ct);
+  assert(ct->usage);
   free(ct->tokenName);
   if (s) ct->tokenName=strdup(s);
   else ct->tokenName=0;
@@ -1754,6 +1763,7 @@ void GWEN_CryptToken_SetTokenName(GWEN_CRYPTTOKEN *ct, const char *s){
 
 const char *GWEN_CryptToken_GetTokenType(const GWEN_CRYPTTOKEN *ct){
   assert(ct);
+  assert(ct->usage);
   return ct->tokenType;
 }
 
@@ -1761,6 +1771,7 @@ const char *GWEN_CryptToken_GetTokenType(const GWEN_CRYPTTOKEN *ct){
 
 const char *GWEN_CryptToken_GetTokenSubType(const GWEN_CRYPTTOKEN *ct){
   assert(ct);
+  assert(ct->usage);
   return ct->tokenSubType;
 }
 
@@ -1769,6 +1780,7 @@ const char *GWEN_CryptToken_GetTokenSubType(const GWEN_CRYPTTOKEN *ct){
 GWEN_CRYPTTOKEN_DEVICE
 GWEN_CryptToken_GetDeviceType(const GWEN_CRYPTTOKEN *ct){
   assert(ct);
+  assert(ct->usage);
   return ct->deviceType;
 }
 
@@ -1776,6 +1788,7 @@ GWEN_CryptToken_GetDeviceType(const GWEN_CRYPTTOKEN *ct){
 
 const char *GWEN_CryptToken_GetDescriptiveName(const GWEN_CRYPTTOKEN *ct) {
   assert(ct);
+  assert(ct->usage);
   return ct->descriptiveName;
 }
 
@@ -1783,6 +1796,7 @@ const char *GWEN_CryptToken_GetDescriptiveName(const GWEN_CRYPTTOKEN *ct) {
 
 void GWEN_CryptToken_SetDescriptiveName(GWEN_CRYPTTOKEN *ct, const char *s) {
   assert(ct);
+  assert(ct->usage);
   free(ct->descriptiveName);
   if (s) ct->descriptiveName=strdup(s);
   else ct->descriptiveName=0;
@@ -1792,6 +1806,7 @@ void GWEN_CryptToken_SetDescriptiveName(GWEN_CRYPTTOKEN *ct, const char *s) {
 
 GWEN_TYPE_UINT32 GWEN_CryptToken_GetFlags(const GWEN_CRYPTTOKEN *ct) {
   assert(ct);
+  assert(ct->usage);
   return ct->flags;
 }
 
@@ -1799,6 +1814,7 @@ GWEN_TYPE_UINT32 GWEN_CryptToken_GetFlags(const GWEN_CRYPTTOKEN *ct) {
 
 void GWEN_CryptToken_SetFlags(GWEN_CRYPTTOKEN *ct, GWEN_TYPE_UINT32 fl) {
   assert(ct);
+  assert(ct->usage);
   ct->flags=fl;
 }
 
@@ -1806,6 +1822,7 @@ void GWEN_CryptToken_SetFlags(GWEN_CRYPTTOKEN *ct, GWEN_TYPE_UINT32 fl) {
 
 void GWEN_CryptToken_AddFlags(GWEN_CRYPTTOKEN *ct, GWEN_TYPE_UINT32 fl) {
   assert(ct);
+  assert(ct->usage);
   ct->flags|=fl;
 }
 
@@ -1813,6 +1830,7 @@ void GWEN_CryptToken_AddFlags(GWEN_CRYPTTOKEN *ct, GWEN_TYPE_UINT32 fl) {
 
 void GWEN_CryptToken_SubFlags(GWEN_CRYPTTOKEN *ct, GWEN_TYPE_UINT32 fl) {
   assert(ct);
+  assert(ct->usage);
   ct->flags&=~fl;
 }
 
@@ -1821,6 +1839,7 @@ void GWEN_CryptToken_SubFlags(GWEN_CRYPTTOKEN *ct, GWEN_TYPE_UINT32 fl) {
 void GWEN_CryptToken_SetOpenFn(GWEN_CRYPTTOKEN *ct,
                                GWEN_CRYPTTOKEN_OPEN_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->openFn=fn;
 }
 
@@ -1829,6 +1848,7 @@ void GWEN_CryptToken_SetOpenFn(GWEN_CRYPTTOKEN *ct,
 GWEN_CRYPTTOKEN_OPEN_FN
 GWEN_CryptToken_GetOpenFn(const GWEN_CRYPTTOKEN *ct) {
   assert(ct);
+  assert(ct->usage);
   return ct->openFn;
 }
 
@@ -1837,6 +1857,7 @@ GWEN_CryptToken_GetOpenFn(const GWEN_CRYPTTOKEN *ct) {
 void GWEN_CryptToken_SetCreateFn(GWEN_CRYPTTOKEN *ct,
                                  GWEN_CRYPTTOKEN_CREATE_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->createFn=fn;
 }
 
@@ -1845,6 +1866,7 @@ void GWEN_CryptToken_SetCreateFn(GWEN_CRYPTTOKEN *ct,
 GWEN_CRYPTTOKEN_CREATE_FN
 GWEN_CryptToken_GetCreateFn(const GWEN_CRYPTTOKEN *ct) {
   assert(ct);
+  assert(ct->usage);
   return ct->createFn;
 }
 
@@ -1853,6 +1875,7 @@ GWEN_CryptToken_GetCreateFn(const GWEN_CRYPTTOKEN *ct) {
 void GWEN_CryptToken_SetCloseFn(GWEN_CRYPTTOKEN *ct,
                                 GWEN_CRYPTTOKEN_CLOSE_FN fn) {
   assert(ct);
+  assert(ct->usage);
   ct->closeFn=fn;
 }
 
@@ -1861,6 +1884,7 @@ void GWEN_CryptToken_SetCloseFn(GWEN_CRYPTTOKEN *ct,
 GWEN_CRYPTTOKEN_CLOSE_FN
 GWEN_CryptToken_GetCloseFn(const GWEN_CRYPTTOKEN *ct) {
   assert(ct);
+  assert(ct->usage);
   return ct->closeFn;
 }
 
@@ -1869,6 +1893,7 @@ GWEN_CryptToken_GetCloseFn(const GWEN_CRYPTTOKEN *ct) {
 void GWEN_CryptToken_SetChangePinFn(GWEN_CRYPTTOKEN *ct,
                                     GWEN_CRYPTTOKEN_CHANGEPIN_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->changePinFn=fn;
 }
 
@@ -1877,6 +1902,7 @@ void GWEN_CryptToken_SetChangePinFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetSignFn(GWEN_CRYPTTOKEN *ct,
                                GWEN_CRYPTTOKEN_SIGN_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->signFn=fn;
 }
 
@@ -1885,6 +1911,7 @@ void GWEN_CryptToken_SetSignFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetVerifyFn(GWEN_CRYPTTOKEN *ct,
                                  GWEN_CRYPTTOKEN_VERIFY_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->verifyFn=fn;
 }
 
@@ -1893,6 +1920,7 @@ void GWEN_CryptToken_SetVerifyFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetEncryptFn(GWEN_CRYPTTOKEN *ct,
                                   GWEN_CRYPTTOKEN_ENCRYPT_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->encryptFn=fn;
 }
 
@@ -1901,6 +1929,7 @@ void GWEN_CryptToken_SetEncryptFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetDecryptFn(GWEN_CRYPTTOKEN *ct,
                                   GWEN_CRYPTTOKEN_DECRYPT_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->decryptFn=fn;
 }
 
@@ -1909,6 +1938,7 @@ void GWEN_CryptToken_SetDecryptFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetReadKeyFn(GWEN_CRYPTTOKEN *ct,
                                   GWEN_CRYPTTOKEN_READKEY_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->readKeyFn=fn;
 }
 
@@ -1917,6 +1947,7 @@ void GWEN_CryptToken_SetReadKeyFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetWriteKeyFn(GWEN_CRYPTTOKEN *ct,
                                    GWEN_CRYPTTOKEN_WRITEKEY_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->writeKeyFn=fn;
 }
 
@@ -1925,6 +1956,7 @@ void GWEN_CryptToken_SetWriteKeyFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetReadKeySpecFn(GWEN_CRYPTTOKEN *ct,
                                       GWEN_CRYPTTOKEN_READKEYSPEC_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->readKeySpecFn=fn;
 }
 
@@ -1933,6 +1965,7 @@ void GWEN_CryptToken_SetReadKeySpecFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetWriteKeySpecFn(GWEN_CRYPTTOKEN *ct,
                                        GWEN_CRYPTTOKEN_WRITEKEYSPEC_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->writeKeySpecFn=fn;
 }
 
@@ -1941,6 +1974,7 @@ void GWEN_CryptToken_SetWriteKeySpecFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetGenerateKeyFn(GWEN_CRYPTTOKEN *ct,
                                       GWEN_CRYPTTOKEN_GENERATEKEY_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->generateKeyFn=fn;
 }
 
@@ -1949,6 +1983,7 @@ void GWEN_CryptToken_SetGenerateKeyFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetFillContextListFn(GWEN_CRYPTTOKEN *ct,
                                           GWEN_CRYPTTOKEN_FILLCONTEXTLIST_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->fillContextListFn=fn;
 }
 
@@ -1957,6 +1992,7 @@ void GWEN_CryptToken_SetFillContextListFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetFillSignInfoListFn(GWEN_CRYPTTOKEN *ct,
                                            GWEN_CRYPTTOKEN_FILLSIGNINFOLIST_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->fillSignInfoListFn=fn;
 }
 
@@ -1965,6 +2001,7 @@ void GWEN_CryptToken_SetFillSignInfoListFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetFillCryptInfoListFn(GWEN_CRYPTTOKEN *ct,
                                             GWEN_CRYPTTOKEN_FILLCRYPTINFOLIST_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->fillCryptInfoListFn=fn;
 }
 
@@ -1973,6 +2010,7 @@ void GWEN_CryptToken_SetFillCryptInfoListFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetFillKeyInfoListFn(GWEN_CRYPTTOKEN *ct,
                                           GWEN_CRYPTTOKEN_FILLKEYINFOLIST_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->fillKeyInfoListFn=fn;
 }
 
@@ -1981,6 +2019,7 @@ void GWEN_CryptToken_SetFillKeyInfoListFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetFillUserListFn(GWEN_CRYPTTOKEN *ct,
                                        GWEN_CRYPTTOKEN_FILLUSERLIST_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->fillUserListFn=fn;
 }
 
@@ -1989,6 +2028,7 @@ void GWEN_CryptToken_SetFillUserListFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetGetSignSeqFn(GWEN_CRYPTTOKEN *ct,
                                      GWEN_CRYPTTOKEN_GETSIGNSEQ_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->getSignSeqFn=fn;
 }
 
@@ -1997,6 +2037,7 @@ void GWEN_CryptToken_SetGetSignSeqFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetModifyUserFn(GWEN_CRYPTTOKEN *ct,
                                      GWEN_CRYPTTOKEN_MODIFYUSER_FN fn) {
   assert(ct);
+  assert(ct->usage);
   ct->modifyUserFn=fn;
 }
 
@@ -2005,6 +2046,7 @@ void GWEN_CryptToken_SetModifyUserFn(GWEN_CRYPTTOKEN *ct,
 void GWEN_CryptToken_SetGetTokenIdDataFn(GWEN_CRYPTTOKEN *ct,
                                          GWEN_CRYPTTOKEN_GETTOKENIDDATA_FN fn){
   assert(ct);
+  assert(ct->usage);
   ct->getTokenIdDataFn=fn;
 }
 
@@ -2015,8 +2057,9 @@ int GWEN_CryptToken_Open(GWEN_CRYPTTOKEN *ct, int manage){
   int rv;
 
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Already open");
+    DBG_WARN(GWEN_LOGDOMAIN, "Already open");
     return GWEN_ERROR_OPEN;
   }
   if (ct->openFn==0)
@@ -2034,8 +2077,9 @@ int GWEN_CryptToken_Create(GWEN_CRYPTTOKEN *ct){
   int rv;
 
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Already open");
+    DBG_WARN(GWEN_LOGDOMAIN, "Already open");
     return GWEN_ERROR_OPEN;
   }
   if (ct->createFn==0)
@@ -2051,8 +2095,9 @@ int GWEN_CryptToken_Create(GWEN_CRYPTTOKEN *ct){
 
 int GWEN_CryptToken_Close(GWEN_CRYPTTOKEN *ct){
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
+    DBG_WARN(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
   }
   if (ct->closeFn==0)
@@ -2065,6 +2110,7 @@ int GWEN_CryptToken_Close(GWEN_CRYPTTOKEN *ct){
 int GWEN_CryptToken_ChangePin(GWEN_CRYPTTOKEN *ct,
 			      GWEN_CRYPTTOKEN_PINTYPE pt){
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2082,6 +2128,7 @@ int GWEN_CryptToken_Sign(GWEN_CRYPTTOKEN *ct,
                          unsigned int len,
                          GWEN_BUFFER *dst){
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2100,6 +2147,7 @@ int GWEN_CryptToken_Verify(GWEN_CRYPTTOKEN *ct,
                            const char *sigptr,
                            unsigned int siglen) {
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2117,6 +2165,7 @@ int GWEN_CryptToken_Encrypt(GWEN_CRYPTTOKEN *ct,
                             unsigned int len,
                             GWEN_BUFFER *dst){
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2134,6 +2183,7 @@ int GWEN_CryptToken_Decrypt(GWEN_CRYPTTOKEN *ct,
                             unsigned int len,
                             GWEN_BUFFER *dst){
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2149,6 +2199,7 @@ int GWEN_CryptToken_ReadKey(GWEN_CRYPTTOKEN *ct,
                             GWEN_TYPE_UINT32 kid,
                             GWEN_CRYPTKEY **key){
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2164,6 +2215,7 @@ int GWEN_CryptToken_WriteKey(GWEN_CRYPTTOKEN *ct,
                              GWEN_TYPE_UINT32 kid,
                              const GWEN_CRYPTKEY *key){
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2179,6 +2231,7 @@ int GWEN_CryptToken_ReadKeySpec(GWEN_CRYPTTOKEN *ct,
                                 GWEN_TYPE_UINT32 kid,
                                 GWEN_KEYSPEC **ks) {
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2194,6 +2247,7 @@ int GWEN_CryptToken_WriteKeySpec(GWEN_CRYPTTOKEN *ct,
                                  GWEN_TYPE_UINT32 kid,
                                  const GWEN_KEYSPEC *ks) {
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2209,6 +2263,7 @@ int GWEN_CryptToken_GetSignSeq(GWEN_CRYPTTOKEN *ct,
                                GWEN_TYPE_UINT32 kid,
                                GWEN_TYPE_UINT32 *signSeq){
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2223,6 +2278,7 @@ int GWEN_CryptToken_GetSignSeq(GWEN_CRYPTTOKEN *ct,
 int GWEN_CryptToken_AddContext(GWEN_CRYPTTOKEN *ct,
                                const GWEN_CRYPTTOKEN_CONTEXT *ctx){
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2239,6 +2295,7 @@ int GWEN_CryptToken_FillContextList(GWEN_CRYPTTOKEN *ct,
   GWEN_CRYPTTOKEN_CONTEXT *ctx;
 
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2270,6 +2327,7 @@ int GWEN_CryptToken_FillSignInfoList(GWEN_CRYPTTOKEN *ct,
   GWEN_CRYPTTOKEN_SIGNINFO *si;
 
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2300,6 +2358,7 @@ int GWEN_CryptToken_FillCryptInfoList(GWEN_CRYPTTOKEN *ct,
   GWEN_CRYPTTOKEN_CRYPTINFO *ci;
 
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2330,6 +2389,7 @@ int GWEN_CryptToken_FillKeyInfoList(GWEN_CRYPTTOKEN *ct,
   GWEN_CRYPTTOKEN_KEYINFO *ki;
 
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2359,6 +2419,7 @@ int GWEN_CryptToken_GenerateKey(GWEN_CRYPTTOKEN *ct,
                                 const GWEN_CRYPTTOKEN_KEYINFO *ki,
                                 GWEN_CRYPTKEY **key){
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2378,6 +2439,7 @@ GWEN_CryptToken_GetSignInfoByAlgos(GWEN_CRYPTTOKEN *ct,
   GWEN_CRYPTTOKEN_SIGNINFO *si;
 
   assert(ct);
+  assert(ct->usage);
 
   if (GWEN_CryptToken_SignInfo_List_GetCount(ct->signInfoList)==0) {
     if (ct->fillSignInfoListFn==0) {
@@ -2414,6 +2476,7 @@ GWEN_CryptToken_GetSignInfoById(GWEN_CRYPTTOKEN *ct,
   GWEN_CRYPTTOKEN_SIGNINFO *si;
 
   assert(ct);
+  assert(ct->usage);
 
   if (GWEN_CryptToken_SignInfo_List_GetCount(ct->signInfoList)==0) {
     if (ct->fillSignInfoListFn==0) {
@@ -2450,6 +2513,7 @@ GWEN_CryptToken_GetCryptInfoByAlgos(GWEN_CRYPTTOKEN *ct,
   GWEN_CRYPTTOKEN_CRYPTINFO *ci;
 
   assert(ct);
+  assert(ct->usage);
 
   if (GWEN_CryptToken_CryptInfo_List_GetCount(ct->cryptInfoList)==0) {
     if (ct->fillCryptInfoListFn==0) {
@@ -2487,6 +2551,7 @@ GWEN_CryptToken_GetCryptInfoById(GWEN_CRYPTTOKEN *ct,
   GWEN_CRYPTTOKEN_CRYPTINFO *ci;
 
   assert(ct);
+  assert(ct->usage);
 
   if (GWEN_CryptToken_CryptInfo_List_GetCount(ct->cryptInfoList)==0) {
     if (ct->fillCryptInfoListFn==0) {
@@ -2522,6 +2587,7 @@ GWEN_CryptToken_GetKeyInfoById(GWEN_CRYPTTOKEN *ct,
   GWEN_CRYPTTOKEN_KEYINFO *ki;
 
   assert(ct);
+  assert(ct->usage);
 
   if (GWEN_CryptToken_KeyInfo_List_GetCount(ct->keyInfoList)==0) {
     if (ct->fillKeyInfoListFn==0) {
@@ -2557,6 +2623,7 @@ GWEN_CryptToken_GetContextById(GWEN_CRYPTTOKEN *ct,
   GWEN_CRYPTTOKEN_CONTEXT *ctx;
 
   assert(ct);
+  assert(ct->usage);
 
   if (GWEN_CryptToken_Context_List_GetCount(ct->contextList)==0) {
     if (ct->fillContextListFn==0) {
@@ -2592,6 +2659,7 @@ int GWEN_CryptToken_FillUserList(GWEN_CRYPTTOKEN *ct,
   GWEN_CRYPTTOKEN_USER *u;
 
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return GWEN_ERROR_NOT_OPEN;
@@ -2629,6 +2697,7 @@ int GWEN_CryptToken_ModifyUser(GWEN_CRYPTTOKEN *ct,
   int i;
 
   assert(ct);
+  assert(ct->usage);
   if (ct->isOpen==0) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Not open");
     return 0;
@@ -2637,7 +2706,7 @@ int GWEN_CryptToken_ModifyUser(GWEN_CRYPTTOKEN *ct,
 
   id=GWEN_CryptToken_User_GetId(u);
   if (id==0) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Invalid id");
+    DBG_ERROR(GWEN_LOGDOMAIN, "Invalid NULL user id");
     return GWEN_ERROR_INVALID;
   }
 
@@ -2650,7 +2719,7 @@ int GWEN_CryptToken_ModifyUser(GWEN_CRYPTTOKEN *ct,
       break;
   }
   if (!ou) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Unknown user");
+    DBG_ERROR(GWEN_LOGDOMAIN, "Unknown user (id=%d)", (int)id);
     return GWEN_ERROR_INVALID;
   }
 
@@ -2658,7 +2727,7 @@ int GWEN_CryptToken_ModifyUser(GWEN_CRYPTTOKEN *ct,
     return GWEN_ERROR_UNSUPPORTED;
   rv=ct->modifyUserFn(ct, u);
   if (rv) {
-    DBG_INFO(GWEN_LOGDOMAIN, "here");
+    DBG_DEBUG(GWEN_LOGDOMAIN, "modityUserFn returned nonzero.");
     return rv;
   }
 
@@ -2691,6 +2760,7 @@ int GWEN_CryptToken_ModifyUser(GWEN_CRYPTTOKEN *ct,
 
 int GWEN_CryptToken_GetTokenIdData(GWEN_CRYPTTOKEN *ct, GWEN_BUFFER *buf) {
   assert(ct);
+  assert(ct->usage);
   assert(buf);
 
   if (ct->getTokenIdDataFn==0) {
@@ -2711,8 +2781,8 @@ int GWEN_CryptToken_Hash(GWEN_CRYPTTOKEN_HASHALGO algo,
   assert(src);
   assert(slen);
 
-  DBG_ERROR(GWEN_LOGDOMAIN, "Hashing with algo \"%s\"",
-            GWEN_CryptToken_HashAlgo_toString(algo));
+  DBG_INFO(GWEN_LOGDOMAIN, "Hashing with algo \"%s\"",
+	   GWEN_CryptToken_HashAlgo_toString(algo));
 
   switch(algo) {
   case GWEN_CryptToken_HashAlgo_None:
@@ -2757,8 +2827,8 @@ int GWEN_CryptToken_Padd(GWEN_CRYPTTOKEN_PADDALGO algo,
   unsigned bsize;
   unsigned dstSize;
 
-  DBG_ERROR(GWEN_LOGDOMAIN, "Padding with algo \"%s\"",
-            GWEN_CryptToken_PaddAlgo_toString(algo));
+  DBG_INFO(GWEN_LOGDOMAIN, "Padding with algo \"%s\"",
+	   GWEN_CryptToken_PaddAlgo_toString(algo));
 
   assert(buf);
   bsize=GWEN_Buffer_GetUsedBytes(buf);
@@ -3293,9 +3363,9 @@ int GWEN_CryptManager_CheckToken(GWEN_PLUGIN_MANAGER *pm,
             logbuffer[sizeof(logbuffer)-1]=0;
             GWEN_WaitCallback_Log(GWEN_LoggerLevelNotice, logbuffer);
 
-            DBG_ERROR(GWEN_LOGDOMAIN,
-                      "Checking plugin \"%s\"",
-                      GWEN_Plugin_GetName(pl));
+            DBG_INFO(GWEN_LOGDOMAIN,
+		     "Checking plugin \"%s\"",
+		     GWEN_Plugin_GetName(pl));
 
             rv=GWEN_CryptToken_Plugin_CheckToken(pl,
                                                  lSubTypeName,
@@ -3503,7 +3573,7 @@ int GWEN_CryptToken__TransformFromFPIN2(unsigned char *buffer,
   unsigned int len;
 
   if (*pinLength<8) {
-    DBG_ERROR(GWEN_LOGDOMAIN, "Pin too small to be a FPIN2 (%d)", *pinLength);
+    DBG_ERROR(GWEN_LOGDOMAIN, "Pin too small to be a FPIN2 (%d<8)", *pinLength);
     return GWEN_ERROR_INVALID;
   }
   len=(buffer[0] & 0x0f);
@@ -3570,7 +3640,7 @@ int GWEN_CryptToken__TransformFromBin(unsigned char *buffer,
 
     c=buffer[i];
     if (c>9) {
-      DBG_ERROR(GWEN_LOGDOMAIN, "Invalid element in pin");
+      DBG_ERROR(GWEN_LOGDOMAIN, "Invalid element in pin (a digit > 9)");
       free(newBuf);
       return GWEN_ERROR_INVALID;
     }
@@ -3606,7 +3676,7 @@ int GWEN_CryptToken__TransformToBCD(unsigned char *buffer,
     /* 1st digit */
     c1=buffer[i];
     if (c1<'0' || c1>'9') {
-      DBG_ERROR(GWEN_LOGDOMAIN, "Invalid element in pin");
+      DBG_ERROR(GWEN_LOGDOMAIN, "Invalid element in pin (a non-number character)");
       free(newBuf);
       return GWEN_ERROR_INVALID;
     }
@@ -3621,7 +3691,7 @@ int GWEN_CryptToken__TransformToBCD(unsigned char *buffer,
     /* 2nd digit */
     c2=buffer[i];
     if (c2<'0' || c2>'9') {
-      DBG_ERROR(GWEN_LOGDOMAIN, "Invalid element in pin");
+      DBG_ERROR(GWEN_LOGDOMAIN, "Invalid element in pin (a non-number character)");
       free(newBuf);
       return GWEN_ERROR_INVALID;
     }
@@ -3680,7 +3750,7 @@ int GWEN_CryptToken__TransformToFPIN2(unsigned char *buffer,
     /* 1st digit */
     c1=buffer[i];
     if (c1<'0' || c1>'9') {
-      DBG_ERROR(GWEN_LOGDOMAIN, "Invalid element in pin");
+      DBG_ERROR(GWEN_LOGDOMAIN, "Invalid element in pin (a non-number character)");
       free(newBuf);
       return GWEN_ERROR_INVALID;
     }
@@ -3694,7 +3764,7 @@ int GWEN_CryptToken__TransformToFPIN2(unsigned char *buffer,
     /* 2nd digit */
     c2=buffer[i];
     if (c2<'0' || c2>'9') {
-      DBG_ERROR(GWEN_LOGDOMAIN, "Invalid element in pin");
+      DBG_ERROR(GWEN_LOGDOMAIN, "Invalid element in pin (a non-number character)");
       free(newBuf);
       return GWEN_ERROR_INVALID;
     }
@@ -3735,7 +3805,7 @@ int GWEN_CryptToken__TransformToBin(unsigned char *buffer,
 
     c=buffer[i];
     if (c<'0' || c>'9') {
-      DBG_ERROR(GWEN_LOGDOMAIN, "Invalid element in pin");
+      DBG_ERROR(GWEN_LOGDOMAIN, "Invalid element in pin (a non-number character)");
       free(newBuf);
       return GWEN_ERROR_INVALID;
     }
