@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: ipc.c,v $
                              -------------------
-    cvs         : $Id: ipc.c,v 1.30 2005/08/04 04:55:46 aquamaniac Exp $
+    cvs         : $Id: ipc.c,v 1.33 2005/10/13 05:22:49 aquamaniac Exp $
     begin       : Fri May 07 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -26,7 +26,7 @@
 
 GWEN_LIST_FUNCTIONS(GWEN_IPCNODE, GWEN_IPCNode)
 GWEN_LIST_FUNCTIONS(GWEN_IPCMSG, GWEN_IPCMsg)
-GWEN_LIST_FUNCTIONS(GWEN_IPCREQUEST, GWEN_IPCRequest)
+GWEN_LIST_FUNCTIONS(GWEN_IPC__REQUEST, GWEN_IPC__Request)
 
 
 static GWEN_TYPE_UINT32 gwen_ipc__lastid=0;
@@ -207,36 +207,48 @@ void GWEN_IPCMsg_Dump(GWEN_IPCMSG *m, FILE *f, int indent){
 
 
 /* -------------------------------------------------------------- FUNCTION */
-GWEN_IPCREQUEST *GWEN_IPCRequest_new(){
-  GWEN_IPCREQUEST *r;
+GWEN_IPC__REQUEST *GWEN_IPC__Request_new(){
+  GWEN_IPC__REQUEST *r;
 
-  GWEN_NEW_OBJECT(GWEN_IPCREQUEST, r);
-  DBG_MEM_INC("GWEN_IPCREQUEST", 0);
-  GWEN_LIST_INIT(GWEN_IPCREQUEST, r);
+  GWEN_NEW_OBJECT(GWEN_IPC__REQUEST, r);
+  DBG_MEM_INC("GWEN_IPC__REQUEST", 0);
+  GWEN_LIST_INIT(GWEN_IPC__REQUEST, r);
 
   r->requestMsgs=GWEN_IPCMsg_List_new();
   r->responseMsgs=GWEN_IPCMsg_List_new();
-  r->id=++gwen_ipc__lastid;
+  r->usage=1;
   return r;
 }
 
 
 
 /* -------------------------------------------------------------- FUNCTION */
-void GWEN_IPCRequest_free(GWEN_IPCREQUEST *r){
+void GWEN_IPC__Request_free(GWEN_IPC__REQUEST *r){
   if (r) {
-    DBG_MEM_DEC("GWEN_IPCREQUEST");
-    GWEN_IPCMsg_List_free(r->responseMsgs);
-    GWEN_IPCMsg_List_free(r->requestMsgs);
-    GWEN_LIST_FINI(GWEN_IPCREQUEST, r);
-    GWEN_FREE_OBJECT(r);
+    assert(r->usage);
+    if (--(r->usage)==0) {
+      DBG_MEM_DEC("GWEN_IPC__REQUEST");
+      GWEN_IPCMsg_List_free(r->responseMsgs);
+      GWEN_IPCMsg_List_free(r->requestMsgs);
+      GWEN_LIST_FINI(GWEN_IPC__REQUEST, r);
+      GWEN_FREE_OBJECT(r);
+    }
   }
 }
 
 
 
 /* -------------------------------------------------------------- FUNCTION */
-void GWEN_IPCRequest_AddRequestMsg(GWEN_IPCREQUEST *r, GWEN_IPCMSG *m){
+void GWEN_IPC__Request_Attach(GWEN_IPC__REQUEST *r){
+  assert(r);
+  assert(r->usage);
+  r->usage++;
+}
+
+
+
+/* -------------------------------------------------------------- FUNCTION */
+void GWEN_IPC__Request_AddRequestMsg(GWEN_IPC__REQUEST *r, GWEN_IPCMSG *m){
   assert(r);
   assert(m);
   GWEN_IPCMsg_List_Add(m, r->requestMsgs);
@@ -245,7 +257,15 @@ void GWEN_IPCRequest_AddRequestMsg(GWEN_IPCREQUEST *r, GWEN_IPCMSG *m){
 
 
 /* -------------------------------------------------------------- FUNCTION */
-void GWEN_IPCRequest_AddResponseMsg(GWEN_IPCREQUEST *r, GWEN_IPCMSG *m){
+GWEN_IPCMSG_LIST *GWEN_IPC__Request_GetRequestMsgList(const GWEN_IPC__REQUEST *r){
+  assert(r);
+  return r->requestMsgs;
+}
+
+
+
+/* -------------------------------------------------------------- FUNCTION */
+void GWEN_IPC__Request_AddResponseMsg(GWEN_IPC__REQUEST *r, GWEN_IPCMSG *m){
   assert(r);
   assert(m);
   GWEN_IPCMsg_List_Add(m, r->responseMsgs);
@@ -254,7 +274,7 @@ void GWEN_IPCRequest_AddResponseMsg(GWEN_IPCREQUEST *r, GWEN_IPCMSG *m){
 
 
 /* -------------------------------------------------------------- FUNCTION */
-int GWEN_IPCRequest_HasRequestMsg(GWEN_IPCREQUEST *r,
+int GWEN_IPC__Request_HasRequestMsg(GWEN_IPC__REQUEST *r,
                                   GWEN_TYPE_UINT32 nid,
 				  GWEN_TYPE_UINT32 id){
   GWEN_IPCMSG *m;
@@ -276,7 +296,7 @@ int GWEN_IPCRequest_HasRequestMsg(GWEN_IPCREQUEST *r,
 
 
 /* -------------------------------------------------------------- FUNCTION */
-void GWEN_IPCRequest_Dump(GWEN_IPCREQUEST *r, FILE *f, int indent){
+void GWEN_IPC__Request_Dump(GWEN_IPC__REQUEST *r, FILE *f, int indent){
   int i;
   GWEN_IPCMSG *m;
 
@@ -341,9 +361,9 @@ GWEN_IPCMANAGER *GWEN_IPCManager_new(){
   DBG_MEM_INC("GWEN_IPCMANAGER", 0);
   mgr->libId=GWEN_Net_GetLibraryId();
   mgr->nodes=GWEN_IPCNode_List_new();
-  mgr->outRequests=GWEN_IPCRequest_List_new();
-  mgr->newInRequests=GWEN_IPCRequest_List_new();
-  mgr->oldInRequests=GWEN_IPCRequest_List_new();
+  mgr->outRequests=GWEN_IPC__Request_List_new();
+  mgr->newInRequests=GWEN_IPC__Request_List_new();
+  mgr->oldInRequests=GWEN_IPC__Request_List_new();
 
   mgr->usage=1;
   return mgr;
@@ -368,9 +388,9 @@ void GWEN_IPCManager_free(GWEN_IPCMANAGER *mgr){
     assert(mgr->usage);
     if (--(mgr->usage)==0) {
       free(mgr->application);
-      GWEN_IPCRequest_List_free(mgr->oldInRequests);
-      GWEN_IPCRequest_List_free(mgr->newInRequests);
-      GWEN_IPCRequest_List_free(mgr->outRequests);
+      GWEN_IPC__Request_List_free(mgr->oldInRequests);
+      GWEN_IPC__Request_List_free(mgr->newInRequests);
+      GWEN_IPC__Request_List_free(mgr->outRequests);
       GWEN_IPCNode_List_free(mgr->nodes);
 
       GWEN_FREE_OBJECT(mgr);
@@ -481,17 +501,17 @@ GWEN_TYPE_UINT32 GWEN_IPCManager_AddClient(GWEN_IPCMANAGER *mgr,
 /* -------------------------------------------------------------- FUNCTION */
 void GWEN_IPCManager__RemoveNodeRequestMessages(GWEN_IPCMANAGER *mgr,
 						GWEN_IPCNODE *n,
-                                                GWEN_IPCREQUEST_LIST *rl,
+                                                GWEN_IPC__REQUEST_LIST *rl,
                                                 const char *msgType) {
-  GWEN_IPCREQUEST *r;
+  GWEN_IPC__REQUEST *r;
 
   /* remove all messages for/of this client from request */
-  r=GWEN_IPCRequest_List_First(rl);
+  r=GWEN_IPC__Request_List_First(rl);
   while(r) {
-    GWEN_IPCREQUEST *rnext;
+    GWEN_IPC__REQUEST *rnext;
     GWEN_IPCMSG *msg;
 
-    rnext=GWEN_IPCRequest_List_Next(r);
+    rnext=GWEN_IPC__Request_List_Next(r);
     msg=GWEN_IPCMsg_List_First(r->requestMsgs);
     while(msg) {
       GWEN_IPCMSG *nextmsg;
@@ -518,8 +538,8 @@ void GWEN_IPCManager__RemoveNodeRequestMessages(GWEN_IPCMANAGER *mgr,
       /* it is, remove the request */
       DBG_INFO(GWEN_LOGDOMAIN, "Removing %s request %08x for/from node %08x",
 	       msgType, r->id, n->id);
-      GWEN_IPCRequest_List_Del(r);
-      GWEN_IPCRequest_free(r);
+      GWEN_IPC__Request_List_Del(r);
+      GWEN_IPC__Request_free(r);
     }
     r=rnext;
   } /* while r */
@@ -564,16 +584,16 @@ int GWEN_IPCManager_RemoveClient(GWEN_IPCMANAGER *mgr,
 
 
 /* -------------------------------------------------------------- FUNCTION */
-GWEN_IPCREQUEST *GWEN_IPCManager__FindRequest(GWEN_IPCMANAGER *mgr,
+GWEN_IPC__REQUEST *GWEN_IPCManager__FindRequest(GWEN_IPCMANAGER *mgr,
                                               GWEN_TYPE_UINT32 rid,
-					      GWEN_IPCREQUEST *r) {
+					      GWEN_IPC__REQUEST *r) {
   assert(mgr);
   assert(r);
 
   while(r) {
     if (r->id==rid)
       return r;
-    r=GWEN_IPCRequest_List_Next(r);
+    r=GWEN_IPC__Request_List_Next(r);
   }
   return 0;
 }
@@ -736,7 +756,7 @@ GWEN_TYPE_UINT32 GWEN_IPCManager_SendRequest(GWEN_IPCMANAGER *mgr,
                                              GWEN_TYPE_UINT32 nid,
                                              GWEN_DB_NODE *req){
   GWEN_IPCNODE *n;
-  GWEN_IPCREQUEST *r;
+  GWEN_IPC__REQUEST *r;
   GWEN_IPCMSG *m;
 
   n=GWEN_IPCNode_List_First(mgr->nodes);
@@ -760,9 +780,10 @@ GWEN_TYPE_UINT32 GWEN_IPCManager_SendRequest(GWEN_IPCMANAGER *mgr,
     return 0;
   }
 
-  r=GWEN_IPCRequest_new();
-  GWEN_IPCRequest_AddRequestMsg(r, m);
-  GWEN_IPCRequest_List_Add(r, mgr->outRequests);
+  r=GWEN_IPC__Request_new();
+  r->id=++gwen_ipc__lastid;
+  GWEN_IPC__Request_AddRequestMsg(r, m);
+  GWEN_IPC__Request_List_Add(r, mgr->outRequests);
   return r->id;
 }
 
@@ -773,7 +794,7 @@ GWEN_TYPE_UINT32 GWEN_IPCManager_SendMultiRequest(GWEN_IPCMANAGER *mgr,
                                                   GWEN_TYPE_UINT32 mark,
                                                   GWEN_DB_NODE *req){
   GWEN_IPCNODE *n;
-  GWEN_IPCREQUEST *r;
+  GWEN_IPC__REQUEST *r;
 
   r=0;
   n=GWEN_IPCNode_List_First(mgr->nodes);
@@ -796,10 +817,11 @@ GWEN_TYPE_UINT32 GWEN_IPCManager_SendMultiRequest(GWEN_IPCMANAGER *mgr,
         }
         else {
           if (r==0) {
-            r=GWEN_IPCRequest_new();
-            GWEN_IPCRequest_List_Add(r, mgr->outRequests);
+            r=GWEN_IPC__Request_new();
+	    r->id=++gwen_ipc__lastid;
+            GWEN_IPC__Request_List_Add(r, mgr->outRequests);
           }
-          GWEN_IPCRequest_AddRequestMsg(r, m);
+          GWEN_IPC__Request_AddRequestMsg(r, m);
         }
       }
     } /* if mark matches */
@@ -819,15 +841,15 @@ GWEN_TYPE_UINT32 GWEN_IPCManager_SendMultiRequest(GWEN_IPCMANAGER *mgr,
 int GWEN_IPCManager_SendResponse(GWEN_IPCMANAGER *mgr,
                                  GWEN_TYPE_UINT32 rid,
                                  GWEN_DB_NODE *rsp){
-  GWEN_IPCREQUEST *r;
+  GWEN_IPC__REQUEST *r;
   GWEN_IPCMSG *m;
   GWEN_IPCMSG *om;
 
-  r=GWEN_IPCRequest_List_First(mgr->oldInRequests);
+  r=GWEN_IPC__Request_List_First(mgr->oldInRequests);
   while(r) {
     if (r->id==rid)
       break;
-    r=GWEN_IPCRequest_List_Next(r);
+    r=GWEN_IPC__Request_List_Next(r);
   } /* while */
   if (!r) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Request %08x not found", rid);
@@ -851,7 +873,7 @@ int GWEN_IPCManager_SendResponse(GWEN_IPCMANAGER *mgr,
     return -1;
   }
 
-  GWEN_IPCRequest_AddResponseMsg(r, m);
+  GWEN_IPC__Request_AddResponseMsg(r, m);
   return 0;
 }
 
@@ -861,24 +883,24 @@ int GWEN_IPCManager_SendResponse(GWEN_IPCMANAGER *mgr,
 int GWEN_IPCManager_RemoveRequest(GWEN_IPCMANAGER *mgr,
                                   GWEN_TYPE_UINT32 rid,
                                   int inOrOut){
-  GWEN_IPCREQUEST *r;
+  GWEN_IPC__REQUEST *r;
 
   if (inOrOut==0)
-    r=GWEN_IPCRequest_List_First(mgr->oldInRequests);
+    r=GWEN_IPC__Request_List_First(mgr->oldInRequests);
   else
-    r=GWEN_IPCRequest_List_First(mgr->outRequests);
+    r=GWEN_IPC__Request_List_First(mgr->outRequests);
   while(r) {
     if (r->id==rid)
       break;
-    r=GWEN_IPCRequest_List_Next(r);
+    r=GWEN_IPC__Request_List_Next(r);
   } /* while */
   if (!r) {
     DBG_ERROR(GWEN_LOGDOMAIN, "%s request %08x not found",
 	      inOrOut?"Outgoing":"Incoming", rid);
     return -1;
   }
-  GWEN_IPCRequest_List_Del(r);
-  GWEN_IPCRequest_free(r);
+  GWEN_IPC__Request_List_Del(r);
+  GWEN_IPC__Request_free(r);
   return 0;
 }
 
@@ -887,12 +909,26 @@ int GWEN_IPCManager_RemoveRequest(GWEN_IPCMANAGER *mgr,
 /* -------------------------------------------------------------- FUNCTION */
 GWEN_TYPE_UINT32 GWEN_IPCManager_GetNextInRequest(GWEN_IPCMANAGER *mgr,
                                                   GWEN_TYPE_UINT32 mark){
-  GWEN_IPCREQUEST *r;
+  GWEN_IPC__REQUEST *r;
+
+  r=GWEN_IPC__Request_List_First(mgr->newInRequests);
+  while(r) {
+    GWEN_IPCMSG *om;
+
+    if (mark==0)
+      break;
+
+    om=GWEN_IPCMsg_List_First(r->requestMsgs);
+    assert(om);
+    assert(om->node);
+    if (om->node->mark==mark)
+      break;
+    r=GWEN_IPC__Request_List_Next(r);
+  }
 
-  r=GWEN_IPCRequest_List_First(mgr->newInRequests);
   if (r) {
-    GWEN_IPCRequest_List_Del(r);
-    GWEN_IPCRequest_List_Add(r, mgr->oldInRequests);
+    GWEN_IPC__Request_List_Del(r);
+    GWEN_IPC__Request_List_Add(r, mgr->oldInRequests);
     return r->id;
   }
   return 0;
@@ -903,14 +939,14 @@ GWEN_TYPE_UINT32 GWEN_IPCManager_GetNextInRequest(GWEN_IPCMANAGER *mgr,
 /* -------------------------------------------------------------- FUNCTION */
 GWEN_DB_NODE *GWEN_IPCManager_GetInRequestData(GWEN_IPCMANAGER *mgr,
                                                GWEN_TYPE_UINT32 rid){
-  GWEN_IPCREQUEST *r;
+  GWEN_IPC__REQUEST *r;
   GWEN_IPCMSG *om;
 
-  r=GWEN_IPCRequest_List_First(mgr->oldInRequests);
+  r=GWEN_IPC__Request_List_First(mgr->oldInRequests);
   while(r) {
     if (r->id==rid)
       break;
-    r=GWEN_IPCRequest_List_Next(r);
+    r=GWEN_IPC__Request_List_Next(r);
   } /* while */
   if (!r) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Request %08x not found", rid);
@@ -946,18 +982,39 @@ GWEN_NETCONNECTION *GWEN_IPCManager_GetConnection(GWEN_IPCMANAGER *mgr,
 
 
 
+/* -------------------------------------------------------------- FUNCTION */
+GWEN_TYPE_UINT32
+GWEN_IPCManager_GetClientForConnection(const GWEN_IPCMANAGER *mgr,
+                                       const GWEN_NETCONNECTION *conn) {
+  GWEN_IPCNODE *n;
+
+  n=GWEN_IPCNode_List_First(mgr->nodes);
+  while(n) {
+    if (n->connection==conn)
+      break;
+    n=GWEN_IPCNode_List_Next(n);
+  } /* while */
+  if (!n) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "No node found for connection");
+    return 0;
+  }
+  return n->id;
+}
+
+
+
 /* -------------------------------------------------------------- FUNCTION */
 GWEN_DB_NODE *GWEN_IPCManager_PeekResponseData(GWEN_IPCMANAGER *mgr,
                                                GWEN_TYPE_UINT32 rid){
-  GWEN_IPCREQUEST *r;
+  GWEN_IPC__REQUEST *r;
   GWEN_IPCMSG *m;
   GWEN_DB_NODE *db;
 
-  r=GWEN_IPCRequest_List_First(mgr->outRequests);
+  r=GWEN_IPC__Request_List_First(mgr->outRequests);
   while(r) {
     if (r->id==rid)
       break;
-    r=GWEN_IPCRequest_List_Next(r);
+    r=GWEN_IPC__Request_List_Next(r);
   } /* while */
   if (!r) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Request %08x not found", rid);
@@ -980,15 +1037,15 @@ GWEN_DB_NODE *GWEN_IPCManager_PeekResponseData(GWEN_IPCMANAGER *mgr,
 /* -------------------------------------------------------------- FUNCTION */
 GWEN_DB_NODE *GWEN_IPCManager_GetResponseData(GWEN_IPCMANAGER *mgr,
                                               GWEN_TYPE_UINT32 rid){
-  GWEN_IPCREQUEST *r;
+  GWEN_IPC__REQUEST *r;
   GWEN_IPCMSG *m;
   GWEN_DB_NODE *db;
 
-  r=GWEN_IPCRequest_List_First(mgr->outRequests);
+  r=GWEN_IPC__Request_List_First(mgr->outRequests);
   while(r) {
     if (r->id==rid)
       break;
-    r=GWEN_IPCRequest_List_Next(r);
+    r=GWEN_IPC__Request_List_Next(r);
   } /* while */
   if (!r) {
     DBG_ERROR(GWEN_LOGDOMAIN, "Request %08x not found", rid);
@@ -1123,16 +1180,16 @@ int GWEN_IPCManager__Collect(GWEN_IPCMANAGER *mgr, int maxMsg) {
 	  else {
 	    /* found a valid body */
 	    if (refId) {
-	      GWEN_IPCREQUEST *r;
+	      GWEN_IPC__REQUEST *r;
 
 	      /* it pretends to be a response, check it */
 	      GWEN_DB_SetIntValue(dbMsg, GWEN_DB_FLAGS_OVERWRITE_VARS,
 				  "ipc/refId", refId);
-	      r=GWEN_IPCRequest_List_First(mgr->outRequests);
+	      r=GWEN_IPC__Request_List_First(mgr->outRequests);
 	      while(r) {
-		if (GWEN_IPCRequest_HasRequestMsg(r, n->id, refId))
+		if (GWEN_IPC__Request_HasRequestMsg(r, n->id, refId))
 		  break;
-		r=GWEN_IPCRequest_List_Next(r);
+		r=GWEN_IPC__Request_List_Next(r);
 	      } /* while r */
 	      if (!r) {
 		DBG_WARN(GWEN_LOGDOMAIN,
@@ -1150,11 +1207,11 @@ int GWEN_IPCManager__Collect(GWEN_IPCMANAGER *mgr, int maxMsg) {
 		m->id=msgId;
 		m->refId=refId;
 		m->receivedTime=time(0);
-		GWEN_IPCRequest_AddResponseMsg(r, m);
+		GWEN_IPC__Request_AddResponseMsg(r, m);
 	      }
 	    }
 	    else {
-	      GWEN_IPCREQUEST *r;
+	      GWEN_IPC__REQUEST *r;
 	      GWEN_IPCMSG *m;
 
 	      /* this is a new incoming request */
@@ -1165,9 +1222,10 @@ int GWEN_IPCManager__Collect(GWEN_IPCMANAGER *mgr, int maxMsg) {
 	      m->id=msgId;
 	      m->refId=refId;
 	      m->receivedTime=time(0);
-	      r=GWEN_IPCRequest_new();
-	      GWEN_IPCRequest_AddRequestMsg(r, m);
-	      GWEN_IPCRequest_List_Add(r, mgr->newInRequests);
+	      r=GWEN_IPC__Request_new();
+	      r->id=++gwen_ipc__lastid;
+	      GWEN_IPC__Request_AddRequestMsg(r, m);
+	      GWEN_IPC__Request_List_Add(r, mgr->newInRequests);
 	    }
 	  } /* if body is valid */
 	} /* if message id is ok */
@@ -1210,14 +1268,14 @@ GWEN_IPCMSG *GWEN_IPCManager__MakeErrorResponse(GWEN_IPCMANAGER *mgr,
 
 /* -------------------------------------------------------------- FUNCTION */
 int GWEN_IPCManager__CheckRequests(GWEN_IPCMANAGER *mgr) {
-  GWEN_IPCREQUEST *r;
+  GWEN_IPC__REQUEST *r;
 
-  r=GWEN_IPCRequest_List_First(mgr->outRequests);
+  r=GWEN_IPC__Request_List_First(mgr->outRequests);
   while(r) {
     GWEN_IPCMSG *m;
-    GWEN_IPCREQUEST *nextr;
+    GWEN_IPC__REQUEST *nextr;
 
-    nextr=GWEN_IPCRequest_List_Next(r);
+    nextr=GWEN_IPC__Request_List_Next(r);
     m=GWEN_IPCMsg_List_First(r->requestMsgs);
     while(m) {
       GWEN_IPCMSG *nextm;
@@ -1403,7 +1461,7 @@ int GWEN_IPCManager_Disconnect(GWEN_IPCMANAGER *mgr, GWEN_TYPE_UINT32 nid){
 void GWEN_IPCManager_Dump(GWEN_IPCMANAGER *mgr, FILE *f, int indent){
   int i;
   GWEN_IPCNODE *n;
-  GWEN_IPCREQUEST *r;
+  GWEN_IPC__REQUEST *r;
 
   assert(mgr);
   for (i=0; i<indent; i++)
@@ -1423,7 +1481,7 @@ void GWEN_IPCManager_Dump(GWEN_IPCMANAGER *mgr, FILE *f, int indent){
   for (i=0; i<indent; i++)
     fprintf(f, " ");
   fprintf(f, "Active Requests  : %ld\n",
-          GWEN_MemoryDebug_GetObjectCount("GWEN_IPCREQUEST"));
+          GWEN_MemoryDebug_GetObjectCount("GWEN_IPC__REQUEST"));
   for (i=0; i<indent; i++)
     fprintf(f, " ");
   fprintf(f, "Application      : %s\n", mgr->application);
@@ -1446,41 +1504,41 @@ void GWEN_IPCManager_Dump(GWEN_IPCMANAGER *mgr, FILE *f, int indent){
   for (i=0; i<indent; i++)
     fprintf(f, " ");
   fprintf(f, "Outgoing Request(s)\n");
-  r=GWEN_IPCRequest_List_First(mgr->outRequests);
+  r=GWEN_IPC__Request_List_First(mgr->outRequests);
   if (!r) {
     for (i=0; i<indent+4; i++)
       fprintf(f, " ");
     fprintf(f, "none\n");
   }
   while(r) {
-    GWEN_IPCRequest_Dump(r, f, indent+4);
-    r=GWEN_IPCRequest_List_Next(r);
+    GWEN_IPC__Request_Dump(r, f, indent+4);
+    r=GWEN_IPC__Request_List_Next(r);
   }
   for (i=0; i<indent; i++)
     fprintf(f, " ");
   fprintf(f, "Unhandled Incoming Request(s)\n");
-  r=GWEN_IPCRequest_List_First(mgr->newInRequests);
+  r=GWEN_IPC__Request_List_First(mgr->newInRequests);
   if (!r) {
     for (i=0; i<indent+4; i++)
       fprintf(f, " ");
     fprintf(f, "none\n");
   }
   while(r) {
-    GWEN_IPCRequest_Dump(r, f, indent+4);
-    r=GWEN_IPCRequest_List_Next(r);
+    GWEN_IPC__Request_Dump(r, f, indent+4);
+    r=GWEN_IPC__Request_List_Next(r);
   }
   for (i=0; i<indent; i++)
     fprintf(f, " ");
   fprintf(f, "Incoming Request(s) in Work\n");
-  r=GWEN_IPCRequest_List_First(mgr->oldInRequests);
+  r=GWEN_IPC__Request_List_First(mgr->oldInRequests);
   if (!r) {
     for (i=0; i<indent+4; i++)
       fprintf(f, " ");
     fprintf(f, "none\n");
   }
   while(r) {
-    GWEN_IPCRequest_Dump(r, f, indent+4);
-    r=GWEN_IPCRequest_List_Next(r);
+    GWEN_IPC__Request_Dump(r, f, indent+4);
+    r=GWEN_IPC__Request_List_Next(r);
   }
 }
 
