@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: netconnection.c,v $
                              -------------------
-    cvs         : $Id: netconnection.c,v 1.29 2005/08/04 04:55:46 aquamaniac Exp $
+    cvs         : $Id: netconnection.c,v 1.31 2005/10/04 17:17:55 aquamaniac Exp $
     begin       : Mon Feb 09 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -47,6 +47,7 @@
 
 
 GWEN_LIST_FUNCTIONS(GWEN_NETCONNECTION, GWEN_NetConnection)
+GWEN_LIST2_FUNCTIONS(GWEN_NETCONNECTION, GWEN_NetConnection)
 GWEN_INHERIT_FUNCTIONS(GWEN_NETCONNECTION)
 
 
@@ -631,7 +632,7 @@ GWEN_NetConnection_WorkIO(GWEN_NETCONNECTION *conn){
   assert(conn);
 
   doneSomething=0;
-  conn->ioFlags=0;
+  /* TEST-20051003 conn->ioFlags=0; */
   startStatus=GWEN_NetTransport_GetStatus(conn->transportLayer);
 
   /* check for disabled connection */
@@ -711,10 +712,13 @@ GWEN_NetConnection_WorkIO(GWEN_NETCONNECTION *conn){
       }
       conn->lastResult=res;
       if (conn->lastResult==GWEN_NetTransportResultWantRead)
-        conn->ioFlags|=GWEN_NETCONNECTION_IOFLAG_WANTREAD;
+          conn->ioFlags|=GWEN_NETCONNECTION_IOFLAG_WANTREAD;
       else if (conn->lastResult==GWEN_NetTransportResultWantWrite) {
-        conn->ioFlags|=GWEN_NETCONNECTION_IOFLAG_WANTWRITE;
+	conn->ioFlags|=GWEN_NETCONNECTION_IOFLAG_WANTWRITE;
       }
+      else /*TEST-20051003 */
+	conn->ioFlags&=~(GWEN_NETCONNECTION_IOFLAG_WANTWRITE |
+			 GWEN_NETCONNECTION_IOFLAG_WANTREAD);
     }
 
     if (!(GWEN_NetTransport_GetFlags(conn->transportLayer) &
@@ -754,6 +758,9 @@ GWEN_NetConnection_WorkIO(GWEN_NETCONNECTION *conn){
         else if (conn->lastResult==GWEN_NetTransportResultWantWrite) {
           conn->ioFlags|=GWEN_NETCONNECTION_IOFLAG_WANTWRITE;
         }
+	else /*TEST-20051003 */
+	  conn->ioFlags&=~(GWEN_NETCONNECTION_IOFLAG_WANTWRITE |
+			   GWEN_NETCONNECTION_IOFLAG_WANTREAD);
       }
     } /* if not EOF met */
   }
