@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: netconnection.h,v $
                              -------------------
-    cvs         : $Id: netconnection.h,v 1.18 2005/08/04 04:55:46 aquamaniac Exp $
+    cvs         : $Id: netconnection.h,v 1.19 2005/09/07 18:49:46 aquamaniac Exp $
     begin       : Mon Feb 09 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -78,6 +78,7 @@ typedef struct GWEN_NETCONNECTION GWEN_NETCONNECTION;
 #endif
 
 #include <gwenhywfar/misc.h>
+#include <gwenhywfar/list2.h>
 #include <gwenhywfar/inherit.h>
 #include <gwenhywfar/nettransport.h>
 #include <gwenhywfar/netmsg.h>
@@ -98,6 +99,7 @@ typedef enum {
 
 
 GWEN_LIST_FUNCTION_LIB_DEFS(GWEN_NETCONNECTION, GWEN_NetConnection, GWENHYWFAR_API)
+GWEN_LIST2_FUNCTION_LIB_DEFS(GWEN_NETCONNECTION, GWEN_NetConnection, GWENHYWFAR_API)
 GWEN_INHERIT_FUNCTION_LIB_DEFS(GWEN_NETCONNECTION, GWENHYWFAR_API)
 /* No trailing semicolon here because this is a macro call */
 
@@ -505,7 +507,7 @@ GWEN_NETMSG *GWEN_NetConnection_GetInMsg(GWEN_NETCONNECTION *conn);
  */
 GWENHYWFAR_API
 GWEN_NETMSG *GWEN_NetConnection_GetInMsg_Wait(GWEN_NETCONNECTION *conn,
-                                             int timeout);
+                                              int timeout);
 
 /**
  * Returns the next message from the input queue (if any) but keeps it in
