@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: db.c,v $
  -------------------
- cvs         : $Id: db.c,v 1.62 2005/08/22 23:10:45 aquamaniac Exp $
+ cvs         : $Id: db.c,v 1.64 2005/09/07 18:49:46 aquamaniac Exp $
  begin       : Tue Sep 09 2003
  copyright   : (C) 2003 by Martin Preuss
  email       : martin@libchipcard.de
@@ -288,7 +288,8 @@ void GWEN_DB_Node_Unlink(GWEN_DB_NODE *n) {
   GWEN_DB_Node_Unlink_UnDirty(n);
   GWEN_DB_ModifyBranchFlagsUp(parent,
 			      GWEN_DB_NODE_FLAGS_DIRTY,
-			      GWEN_DB_NODE_FLAGS_DIRTY);
+                              GWEN_DB_NODE_FLAGS_DIRTY);
+  n->h.parent=0;
 }
 
 
@@ -2696,6 +2697,50 @@ GWEN_DB_NODE *GWEN_DB_FindNextGroup(GWEN_DB_NODE *n, const char *name){
 
 
 
+GWEN_DB_NODE *GWEN_DB_FindFirstVar(GWEN_DB_NODE *n, const char *name) {
+  GWEN_DB_NODE *nn;
+
+  assert(n);
+  if (n->h.typ!=GWEN_DB_NODETYPE_GROUP) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Node is not a group");
+    return 0;
+  }
+  nn=n->h.child;
+  while(nn) {
+    if (nn->h.typ==GWEN_DB_NODETYPE_VAR) {
+      if (-1!=GWEN_Text_ComparePattern(nn->var.name, name, 0))
+        break;
+    }
+    nn=nn->h.next;
+  } /* while node */
+  return nn;
+}
+
+
+
+GWEN_DB_NODE *GWEN_DB_FindNextVar(GWEN_DB_NODE *n, const char *name) {
+  GWEN_DB_NODE *og;
+
+  og=n;
+  assert(n);
+  if (n->h.typ!=GWEN_DB_NODETYPE_VAR) {
+    DBG_ERROR(GWEN_LOGDOMAIN, "Node is not a group");
+    return 0;
+  }
+  n=n->h.next;
+  while(n) {
+    if (n->h.typ==GWEN_DB_NODETYPE_VAR) {
+      if (-1!=GWEN_Text_ComparePattern(n->var.name, name, 0))
+        break;
+    }
+    n=n->h.next;
+  } /* while node */
+  assert(n!=og);
+  return n;
+}
+
+
+
 const char *GWEN_DB_VariableName(GWEN_DB_NODE *n){
   assert(n);
   if (n->h.typ!=GWEN_DB_NODETYPE_VAR) {
