@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile: code_c.c,v $
                              -------------------
-    cvs         : $Id: code_c.c,v 1.17 2005/07/09 13:38:17 aquamaniac Exp $
+    cvs         : $Id: code_c.c,v 1.20 2005/10/22 15:10:47 aquamaniac Exp $
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -205,6 +205,9 @@ int write_code_freeElem_c(ARGUMENTS *args,
   if (atoi(get_property(node, "ptr", "0"))==0)
     return 0;
 
+  if (atoi(GWEN_XMLNode_GetProperty(node, "takeOver", "0"))==0)
+    return 0;
+
   typ=GWEN_XMLNode_GetProperty(node, "type", 0);
   if (!typ) {
     DBG_ERROR(0, "No type for element");
@@ -271,8 +274,9 @@ int write_code_freeElems_c(ARGUMENTS *args,
 
       if (strcasecmp(GWEN_XMLNode_GetData(n), "group")==0)
         rv=write_code_freeElems_c(args, n, bio);
-      else if (strcasecmp(GWEN_XMLNode_GetData(n), "elem")==0)
-        rv=write_code_freeElem_c(args, n, bio);
+      else if (strcasecmp(GWEN_XMLNode_GetData(n), "elem")==0) {
+	rv=write_code_freeElem_c(args, n, bio);
+      }
       else {
         rv=0;
       }
@@ -350,12 +354,6 @@ int write_code_todbArg_c(ARGUMENTS *args,
   const char *name;
   const char *mode;
   int isPtr;
-  int isVolatile;
-
-  isVolatile=atoi(GWEN_XMLNode_GetProperty(node, "volatile", "0"));
-  if (isVolatile)
-    /* don't save volatile data */
-    return 0;
 
   isPtr=atoi(get_property(node, "ptr", "0"));
 
@@ -722,6 +720,12 @@ int write_code_fromdbArg_c(ARGUMENTS *args,
   const char *mode;
   int isPtr;
   const char *defval;
+  int isVolatile;
+
+  isVolatile=atoi(GWEN_XMLNode_GetProperty(node, "volatile", "0"));
+  if (isVolatile)
+    /* don't save volatile data */
+    return 0;
 
   isPtr=atoi(get_property(node, "ptr", "0"));
 
@@ -976,6 +980,8 @@ int write_code_constrec_c(ARGUMENTS *args,
 	  }
 	}
       }
+      else if (strcasecmp(GWEN_XMLNode_GetData(n), "func")==0) {
+      }
     }
     n=GWEN_XMLNode_GetNextTag(n);
   }
@@ -988,7 +994,6 @@ int write_code_constrec_c(ARGUMENTS *args,
 int write_code_const_c(ARGUMENTS *args,
                        GWEN_XMLNODE *node,
                        GWEN_BUFFEREDIO *bio){
-  GWEN_ERRORCODE err;
   const char *prefix;
   const char *styp;
   int rv;
@@ -1004,61 +1009,41 @@ int write_code_const_c(ARGUMENTS *args,
     return -1;
   }
 
-  err=GWEN_BufferedIO_Write(bio, styp);
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-  err=GWEN_BufferedIO_Write(bio, " *");
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-  err=GWEN_BufferedIO_Write(bio, prefix);
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-  err=GWEN_BufferedIO_WriteLine(bio, "_new() {");
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
+  GWEN_BufferedIO_Write(bio, styp);
+  GWEN_BufferedIO_Write(bio, " *");
+  GWEN_BufferedIO_Write(bio, prefix);
+  GWEN_BufferedIO_WriteLine(bio, "_new() {");
 
-  err=GWEN_BufferedIO_Write(bio, "  ");
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-  err=GWEN_BufferedIO_Write(bio, styp);
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-  err=GWEN_BufferedIO_WriteLine(bio, " *st;");
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-  err=GWEN_BufferedIO_WriteLine(bio, "");
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
+  GWEN_BufferedIO_Write(bio, "  ");
+  GWEN_BufferedIO_Write(bio, styp);
+  GWEN_BufferedIO_WriteLine(bio, " *st;");
+  GWEN_BufferedIO_WriteLine(bio, "");
 
-  err=GWEN_BufferedIO_Write(bio, "  GWEN_NEW_OBJECT(");
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-  err=GWEN_BufferedIO_Write(bio, styp);
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-  err=GWEN_BufferedIO_WriteLine(bio, ", st)");
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-  err=GWEN_BufferedIO_WriteLine(bio, "  st->_usage=1;");
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
+  GWEN_BufferedIO_Write(bio, "  GWEN_NEW_OBJECT(");
+  GWEN_BufferedIO_Write(bio, styp);
+  GWEN_BufferedIO_WriteLine(bio, ", st)");
+  GWEN_BufferedIO_WriteLine(bio, "  st->_usage=1;");
 
   // add inherit functions
   if (get_struct_property(node, "inherit", 0)) {
-    err=GWEN_BufferedIO_Write(bio, "  GWEN_INHERIT_INIT(");
-    if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-    err=GWEN_BufferedIO_Write(bio, styp);
-    if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-    err=GWEN_BufferedIO_WriteLine(bio, ", st)");
-    if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
+    GWEN_BufferedIO_Write(bio, "  GWEN_INHERIT_INIT(");
+    GWEN_BufferedIO_Write(bio, styp);
+    GWEN_BufferedIO_WriteLine(bio, ", st)");
   }
 
   // add list functions
   if (get_struct_property(node, "list", 0)) {
-    err=GWEN_BufferedIO_Write(bio, "  GWEN_LIST_INIT(");
-    if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-    err=GWEN_BufferedIO_Write(bio, styp);
-    if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-    err=GWEN_BufferedIO_WriteLine(bio, ", st)");
-    if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
+    GWEN_BufferedIO_Write(bio, "  GWEN_LIST_INIT(");
+    GWEN_BufferedIO_Write(bio, styp);
+    GWEN_BufferedIO_WriteLine(bio, ", st)");
   }
 
   rv=write_code_constrec_c(args, node, bio);
   if (rv)
     return rv;
 
-  err=GWEN_BufferedIO_WriteLine(bio, "  return st;");
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
-  err=GWEN_BufferedIO_WriteLine(bio, "}");
-  if (!GWEN_Error_IsOk(err)) { DBG_ERROR_ERR(0, err); return -1;}
+  GWEN_BufferedIO_WriteLine(bio, "  return st;");
+  GWEN_BufferedIO_WriteLine(bio, "}");
 
   return 0;
 }
@@ -1229,11 +1214,10 @@ int write_code_setget_c(ARGUMENTS *args,
         GWEN_BufferedIO_Write(bio, name+1);
         GWEN_BufferedIO_WriteLine(bio, ";");
         GWEN_BufferedIO_WriteLine(bio, "}");
+        GWEN_BufferedIO_WriteLine(bio, "");
+        GWEN_BufferedIO_WriteLine(bio, "");
 
         /* write setter */
-        GWEN_BufferedIO_WriteLine(bio, "");
-        GWEN_BufferedIO_WriteLine(bio, "");
-
         GWEN_BufferedIO_Write(bio, "void ");
         GWEN_BufferedIO_Write(bio, prefix);
         GWEN_BufferedIO_Write(bio, "_Set");
@@ -1324,6 +1308,8 @@ int write_code_setget_c(ARGUMENTS *args,
         }
         GWEN_BufferedIO_WriteLine(bio, "  st->_modified=1;");
         GWEN_BufferedIO_WriteLine(bio, "}");
+        GWEN_BufferedIO_WriteLine(bio, "");
+        GWEN_BufferedIO_WriteLine(bio, "");
 
         if (strcasecmp(typ, "GWEN_STRINGLIST")==0) {
           /* special functions for string lists */
@@ -1345,10 +1331,10 @@ int write_code_setget_c(ARGUMENTS *args,
           GWEN_BufferedIO_WriteLine(bio, ", d, 0, chk))");
           GWEN_BufferedIO_WriteLine(bio, "    st->_modified=1;");
           GWEN_BufferedIO_WriteLine(bio, "}");
+          GWEN_BufferedIO_WriteLine(bio, "");
+          GWEN_BufferedIO_WriteLine(bio, "");
 
           /* remove */
-          GWEN_BufferedIO_WriteLine(bio, "");
-          GWEN_BufferedIO_WriteLine(bio, "");
           GWEN_BufferedIO_Write(bio, "void ");
           GWEN_BufferedIO_Write(bio, prefix);
           GWEN_BufferedIO_Write(bio, "_Remove");
@@ -1363,10 +1349,10 @@ int write_code_setget_c(ARGUMENTS *args,
           GWEN_BufferedIO_WriteLine(bio, ", d))");
           GWEN_BufferedIO_WriteLine(bio, "    st->_modified=1;");
           GWEN_BufferedIO_WriteLine(bio, "}");
+          GWEN_BufferedIO_WriteLine(bio, "");
+          GWEN_BufferedIO_WriteLine(bio, "");
 
           /* clear */
-          GWEN_BufferedIO_WriteLine(bio, "");
-          GWEN_BufferedIO_WriteLine(bio, "");
           GWEN_BufferedIO_Write(bio, "void ");
           GWEN_BufferedIO_Write(bio, prefix);
           GWEN_BufferedIO_Write(bio, "_Clear");
@@ -1386,10 +1372,10 @@ int write_code_setget_c(ARGUMENTS *args,
           GWEN_BufferedIO_WriteLine(bio, "    st->_modified=1;");
           GWEN_BufferedIO_WriteLine(bio, "  }");
           GWEN_BufferedIO_WriteLine(bio, "}");
+          GWEN_BufferedIO_WriteLine(bio, "");
+          GWEN_BufferedIO_WriteLine(bio, "");
 
           /* has */
-          GWEN_BufferedIO_WriteLine(bio, "");
-          GWEN_BufferedIO_WriteLine(bio, "");
           GWEN_BufferedIO_Write(bio, "int ");
           GWEN_BufferedIO_Write(bio, prefix);
           GWEN_BufferedIO_Write(bio, "_Has");
@@ -1403,8 +1389,167 @@ int write_code_setget_c(ARGUMENTS *args,
           GWEN_BufferedIO_Write(bio, name+1);
           GWEN_BufferedIO_WriteLine(bio, ", d);");
           GWEN_BufferedIO_WriteLine(bio, "}");
+          GWEN_BufferedIO_WriteLine(bio, "");
+          GWEN_BufferedIO_WriteLine(bio, "");
+        }
+      }
+      else if (strcasecmp(GWEN_XMLNode_GetData(n), "func")==0) {
+        const char *typ;
+        const char *name;
+        const char *rettype;
+        const char *defret;
+        GWEN_XMLNODE *anode;
+        int isPtr;
+        int isVoid;
+        int idx;
+
+        name=GWEN_XMLNode_GetProperty(n, "name", 0);
+        if (!name) {
+          DBG_ERROR(0, "No name for element");
+          return -1;
         }
 
+        typ=GWEN_XMLNode_GetProperty(n, "type", 0);
+        if (!typ) {
+          DBG_ERROR(0, "No type for element");
+          return -1;
+        }
+
+        rettype=GWEN_XMLNode_GetProperty(n, "return", 0);
+        if (!rettype) {
+          DBG_ERROR(0, "No return type for function");
+          return -1;
+        }
+
+        isPtr=atoi(get_property(n, "ptr", "0"));
+        isVoid=(!isPtr && strcasecmp(rettype, "void")==0);
+
+        defret=GWEN_XMLNode_GetProperty(n, "default", 0);
+        if (!defret && !isVoid) {
+          DBG_ERROR(0, "No default return value for function %s", name);
+          return -1;
+        }
+
+        /* getter */
+        GWEN_BufferedIO_Write(bio, styp);
+        GWEN_BufferedIO_Write(bio, "_");
+        GWEN_BufferedIO_Write(bio, typ);
+        GWEN_BufferedIO_Write(bio, " ");
+        GWEN_BufferedIO_Write(bio, prefix);
+        GWEN_BufferedIO_Write(bio, "_Get");
+        GWEN_BufferedIO_WriteChar(bio, toupper(*name));
+        GWEN_BufferedIO_Write(bio, name+1);
+        GWEN_BufferedIO_Write(bio, "(const ");
+        GWEN_BufferedIO_Write(bio, styp);
+        GWEN_BufferedIO_WriteLine(bio, " *st) {");
+        GWEN_BufferedIO_WriteLine(bio, "  assert(st);");
+        GWEN_BufferedIO_Write(bio, "  return st->");
+        GWEN_BufferedIO_WriteChar(bio, tolower(*name));
+        GWEN_BufferedIO_Write(bio, name+1);
+        GWEN_BufferedIO_WriteLine(bio, ";");
+        GWEN_BufferedIO_WriteLine(bio, "}");
+        GWEN_BufferedIO_WriteLine(bio, "");
+        GWEN_BufferedIO_WriteLine(bio, "");
+
+        /* setter */
+        GWEN_BufferedIO_Write(bio, "void ");
+        GWEN_BufferedIO_Write(bio, prefix);
+        GWEN_BufferedIO_Write(bio, "_Set");
+        GWEN_BufferedIO_WriteChar(bio, toupper(*name));
+        GWEN_BufferedIO_Write(bio, name+1);
+        GWEN_BufferedIO_Write(bio, "(");
+        GWEN_BufferedIO_Write(bio, styp);
+        GWEN_BufferedIO_Write(bio, " *st, ");
+        GWEN_BufferedIO_Write(bio, styp);
+        GWEN_BufferedIO_Write(bio, "_");
+        GWEN_BufferedIO_Write(bio, typ);
+        GWEN_BufferedIO_WriteLine(bio, " d) {");
+        GWEN_BufferedIO_WriteLine(bio, "  assert(st);");
+        GWEN_BufferedIO_Write(bio, "  st->");
+        GWEN_BufferedIO_WriteChar(bio, tolower(*name));
+        GWEN_BufferedIO_Write(bio, name+1);
+        GWEN_BufferedIO_WriteLine(bio, "=d;");
+        GWEN_BufferedIO_WriteLine(bio, "}");
+        GWEN_BufferedIO_WriteLine(bio, "");
+        GWEN_BufferedIO_WriteLine(bio, "");
+
+        /* function call */
+        GWEN_BufferedIO_Write(bio, rettype);
+        if (isPtr)
+          GWEN_BufferedIO_Write(bio, "*");
+        GWEN_BufferedIO_Write(bio, " ");
+        GWEN_BufferedIO_Write(bio, prefix);
+        GWEN_BufferedIO_Write(bio, "_");
+        GWEN_BufferedIO_WriteChar(bio, toupper(*name));
+        GWEN_BufferedIO_Write(bio, name+1);
+        GWEN_BufferedIO_Write(bio, "(");
+
+        GWEN_BufferedIO_Write(bio, styp);
+        GWEN_BufferedIO_Write(bio, " *st");
+
+        anode=GWEN_XMLNode_FindFirstTag(n, "arg", 0, 0);
+        idx=0;
+        while(anode) {
+          const char *aname;
+          const char *atype;
+          int aisPtr;
+    
+          GWEN_BufferedIO_Write(bio, ", ");
+
+          aisPtr=atoi(GWEN_XMLNode_GetProperty(anode, "ptr", "0"));
+          aname=GWEN_XMLNode_GetProperty(anode, "name", 0);
+          if (!aname || !*aname) {
+            DBG_ERROR(0, "No name for argument %d in function %s", idx, name);
+            return -1;
+          }
+          atype=GWEN_XMLNode_GetProperty(anode, "type", 0);
+          if (!atype || !*atype) {
+            DBG_ERROR(0, "No type for argument %d in function %s", idx, name);
+            return -1;
+          }
+    
+          GWEN_BufferedIO_Write(bio, atype);
+          if (aisPtr)
+            GWEN_BufferedIO_Write(bio, "*");
+          GWEN_BufferedIO_Write(bio, " ");
+          GWEN_BufferedIO_Write(bio, aname);
+    
+          idx++;
+          anode=GWEN_XMLNode_FindNextTag(anode, "arg", 0, 0);
+        }
+
+        GWEN_BufferedIO_WriteLine(bio, ") {");
+        GWEN_BufferedIO_WriteLine(bio, "  assert(st);");
+        GWEN_BufferedIO_Write(bio, "  if (st->");
+        GWEN_BufferedIO_WriteChar(bio, tolower(*name));
+        GWEN_BufferedIO_Write(bio, name+1);
+        GWEN_BufferedIO_WriteLine(bio, ")");
+        GWEN_BufferedIO_Write(bio, "    ");
+        if (!isVoid)
+          GWEN_BufferedIO_Write(bio, "return ");
+        GWEN_BufferedIO_Write(bio, "st->");
+        GWEN_BufferedIO_WriteChar(bio, tolower(*name));
+        GWEN_BufferedIO_Write(bio, name+1);
+        GWEN_BufferedIO_Write(bio, "(st");
+
+        anode=GWEN_XMLNode_FindFirstTag(n, "arg", 0, 0);
+        while(anode) {
+          const char *aname;
+
+          GWEN_BufferedIO_Write(bio, ", ");
+          aname=GWEN_XMLNode_GetProperty(anode, "name", 0);
+          GWEN_BufferedIO_Write(bio, aname);
+          anode=GWEN_XMLNode_FindNextTag(anode, "arg", 0, 0);
+        }
+        GWEN_BufferedIO_WriteLine(bio, ");");
+        if (!isVoid) {
+          GWEN_BufferedIO_Write(bio, "return ");
+          GWEN_BufferedIO_Write(bio, defret);
+          GWEN_BufferedIO_WriteLine(bio, ";");
+        }
+        GWEN_BufferedIO_WriteLine(bio, "}");
+        GWEN_BufferedIO_WriteLine(bio, "");
+        GWEN_BufferedIO_WriteLine(bio, "");
       }
     }
     n=GWEN_XMLNode_GetNextTag(n);
@@ -1834,33 +1979,39 @@ int write_code_todbrec_c(ARGUMENTS *args, GWEN_XMLNODE *node,
         }
       }
       else if (strcasecmp(GWEN_XMLNode_GetData(n), "elem")==0) {
-        int isPtr;
-        const char *typ;
-        const char *name;
+        int isVolatile;
+      
+        isVolatile=atoi(GWEN_XMLNode_GetProperty(n, "volatile", "0"));
+        if (isVolatile==0) {
+	  int isPtr;
+	  const char *typ;
+	  const char *name;
 
-        name=GWEN_XMLNode_GetProperty(n, "name", 0);
-        if (!name) {
-          DBG_ERROR(0, "No name for element");
-          return -1;
-        }
-
-        typ=GWEN_XMLNode_GetProperty(n, "type", 0);
-        if (!typ) {
-          DBG_ERROR(0, "No type for element");
-          return -1;
-        }
-
-        isPtr=atoi(get_property(n, "ptr", "0"));
-        if (isPtr) {
-          GWEN_BufferedIO_Write(bio, "  if (st->");
-          GWEN_BufferedIO_WriteChar(bio, tolower(*name));
-          GWEN_BufferedIO_Write(bio, name+1);
-          GWEN_BufferedIO_WriteLine(bio, ")");
-        }
-        rv=write_code_todbArg_c(args, n, bio);
-	if (rv) {
-          DBG_ERROR(0, "Error in toDb function");
-	  return rv;
+	  name=GWEN_XMLNode_GetProperty(n, "name", 0);
+	  if (!name) {
+	    DBG_ERROR(0, "No name for element");
+	    return -1;
+	  }
+  
+	  typ=GWEN_XMLNode_GetProperty(n, "type", 0);
+	  if (!typ) {
+	    DBG_ERROR(0, "No type for element");
+	    return -1;
+	  }
+  
+	  isPtr=atoi(get_property(n, "ptr", "0"));
+	  if (isPtr) {
+	    GWEN_BufferedIO_Write(bio, "  if (st->");
+	    GWEN_BufferedIO_WriteChar(bio, tolower(*name));
+	    GWEN_BufferedIO_Write(bio, name+1);
+	    GWEN_BufferedIO_WriteLine(bio, ")");
+	  }
+  
+	  rv=write_code_todbArg_c(args, n, bio);
+	  if (rv) {
+	    DBG_ERROR(0, "Error in toDb function");
+	    return rv;
+	  }
 	}
       }
     }
@@ -1945,281 +2096,298 @@ int write_code_fromdbrec_c(ARGUMENTS *args, GWEN_XMLNODE *node,
         }
       }
       else if (strcasecmp(GWEN_XMLNode_GetData(n), "elem")==0) {
-        int isPtr;
-        const char *typ;
-	const char *name;
-        const char *mode;
+        int isVolatile;
+      
+        isVolatile=atoi(GWEN_XMLNode_GetProperty(n, "volatile", "0"));
+        if (isVolatile==0) {
+	  int isPtr;
+	  const char *typ;
+	  const char *name;
+	  const char *mode;
 
-        name=GWEN_XMLNode_GetProperty(n, "name", 0);
-        if (!name) {
-          DBG_ERROR(0, "No name for element");
-          return -1;
-        }
-
-	mode=GWEN_XMLNode_GetProperty(n, "mode", "single");
-
-        typ=GWEN_XMLNode_GetProperty(n, "type", 0);
-        if (!typ) {
-          DBG_ERROR(0, "No type for element");
-          return -1;
-        }
-
-	if (strcasecmp(mode, "list")==0) {
-	  const char *prefix;
-	  const char *elemType;
-	  const char *elemPrefix;
-	  GWEN_XMLNODE *elemNode;
-
-	  prefix=get_struct_property(node, "prefix", 0);
-	  assert(prefix);
-  
-	  /* create list code */
-	  elemType=GWEN_XMLNode_GetProperty(n, "elemType", 0);
-	  if (!elemType) {
-	    DBG_ERROR(0, "No \"type\" for list type \"%s\"", typ);
+	  name=GWEN_XMLNode_GetProperty(n, "name", 0);
+	  if (!name) {
+	    DBG_ERROR(0, "No name for element");
 	    return -1;
 	  }
   
-	  elemNode=get_typedef(n, elemType);
-	  if (!elemNode) {
-	    DBG_ERROR(0, "Undefined type %s", elemType);
-	    return -1;
-	  }
-	  elemPrefix=GWEN_XMLNode_GetProperty(elemNode, "prefix", 0);
-	  if (!elemPrefix) {
-	    DBG_ERROR(0, "No \"prefix\" for type \"%s\" (within %s)",
-		      elemType, typ);
+	  mode=GWEN_XMLNode_GetProperty(n, "mode", "single");
+  
+	  typ=GWEN_XMLNode_GetProperty(n, "type", 0);
+	  if (!typ) {
+	    DBG_ERROR(0, "No type for element");
 	    return -1;
 	  }
   
-	  /* actually generate the code */
-	  GWEN_BufferedIO_Write(bio, "  st->");
-	  GWEN_BufferedIO_WriteChar(bio, tolower(*name));
-	  GWEN_BufferedIO_Write(bio, name+1);
-	  GWEN_BufferedIO_Write(bio, "=");
-	  GWEN_BufferedIO_Write(bio, elemPrefix);
-	  GWEN_BufferedIO_WriteLine(bio, "_List_new();");
-
-	  GWEN_BufferedIO_WriteLine(bio, "  if (1) {");
-	  GWEN_BufferedIO_WriteLine(bio, "    GWEN_DB_NODE *dbT;");
-	  GWEN_BufferedIO_Write(bio, "    ");
-	  GWEN_BufferedIO_Write(bio, elemType);
-	  GWEN_BufferedIO_WriteLine(bio, " *e;");
-	  GWEN_BufferedIO_WriteLine(bio, "");
-	  GWEN_BufferedIO_Write(bio,
-				"    dbT=GWEN_DB_GetGroup(db, "
-				"GWEN_PATH_FLAGS_NAMEMUSTEXIST, \"");
-	  GWEN_BufferedIO_WriteChar(bio, tolower(*name));
-	  GWEN_BufferedIO_Write(bio, name+1);
-	  GWEN_BufferedIO_WriteLine(bio, "\");");
-	  GWEN_BufferedIO_WriteLine(bio, "    if (dbT) {");
-	  GWEN_BufferedIO_WriteLine(bio, "      GWEN_DB_NODE *dbT2;");
-	  GWEN_BufferedIO_WriteLine(bio, "");
+	  if (strcasecmp(mode, "list")==0) {
+	    const char *prefix;
+	    const char *elemType;
+	    const char *elemPrefix;
+	    GWEN_XMLNODE *elemNode;
   
-	  GWEN_BufferedIO_Write(bio,
-				"      dbT2=GWEN_DB_FindFirstGroup(dbT, \"");
-	  GWEN_BufferedIO_Write(bio, "element");
-	  GWEN_BufferedIO_WriteLine(bio, "\");");
-  
-	  /* while (e) */
-	  GWEN_BufferedIO_WriteLine(bio, "      while(dbT2) {");
-  
-	  /* e=ElemType_fromDb(e) */
-	  GWEN_BufferedIO_Write(bio, "        e=");
-	  GWEN_BufferedIO_Write(bio, elemPrefix);
-	  GWEN_BufferedIO_WriteLine(bio, "_fromDb(dbT2);");
-  
-	  /* if (!e) */
-	  GWEN_BufferedIO_WriteLine(bio, "        if (!e) {");
-	  GWEN_BufferedIO_Write(bio, "          "
-                                "DBG_ERROR(0, \"Bad element for type \\\"");
-          GWEN_BufferedIO_Write(bio, elemType);
-          GWEN_BufferedIO_WriteLine(bio, "\\\"\");");
-	  GWEN_BufferedIO_WriteLine(bio, "          "
-				    "if (GWEN_Logger_GetLevel(0)>="
-				    "GWEN_LoggerLevelDebug)");
-	  GWEN_BufferedIO_WriteLine(bio, "            "
-				    "GWEN_DB_Dump(dbT2, stderr, 2);");
-	  GWEN_BufferedIO_Write(bio, "          ");
-	  GWEN_BufferedIO_Write(bio, prefix);
-	  GWEN_BufferedIO_WriteLine(bio, "_free(st);");
-	  GWEN_BufferedIO_WriteLine(bio, "          return 0;");
-	  GWEN_BufferedIO_WriteLine(bio, "        }");
-  
-	  /* ElemType_List_Add(e, st->NAME); */
-	  GWEN_BufferedIO_Write(bio, "        ");
-	  GWEN_BufferedIO_Write(bio, elemPrefix);
-	  GWEN_BufferedIO_Write(bio, "_List_Add(e, st->");
-	  GWEN_BufferedIO_WriteChar(bio, tolower(*name));
-	  GWEN_BufferedIO_Write(bio, name+1);
-	  GWEN_BufferedIO_Write(bio, ");");
-  
-	  GWEN_BufferedIO_Write(bio,
-				"    dbT2=GWEN_DB_FindNextGroup(dbT2, \"");
-	  GWEN_BufferedIO_Write(bio, "element");
-	  GWEN_BufferedIO_WriteLine(bio, "\");");
-  
-	  GWEN_BufferedIO_WriteLine(bio, "      } /* while */");
-  
-	  GWEN_BufferedIO_WriteLine(bio, "    } /* if (dbT) */");
-  
-	  GWEN_BufferedIO_WriteLine(bio, "  } /* if (1) */");
-	}
-	else if (strcasecmp(mode, "list2")==0) {
-	  const char *prefix;
-	  const char *elemType;
-	  const char *elemPrefix;
-	  GWEN_XMLNODE *elemNode;
-
-	  prefix=get_struct_property(node, "prefix", 0);
-	  assert(prefix);
-  
-	  /* create list code */
-	  elemType=GWEN_XMLNode_GetProperty(n, "elemType", 0);
-	  if (!elemType) {
-	    DBG_ERROR(0, "No \"type\" for list type \"%s\"", typ);
-	    return -1;
-	  }
-  
-	  elemNode=get_typedef(node, elemType);
-	  if (!elemNode) {
-	    DBG_ERROR(0, "Undefined type %s", elemType);
-	    return -1;
-	  }
-	  elemPrefix=GWEN_XMLNode_GetProperty(elemNode, "prefix", 0);
-	  if (!elemPrefix) {
-	    DBG_ERROR(0, "No \"prefix\" for type \"%s\" (within %s)",
-		      elemType, typ);
-	    return -1;
-	  }
-  
-	  /* actually generate the code */
-	  GWEN_BufferedIO_Write(bio, "  st->");
-	  GWEN_BufferedIO_WriteChar(bio, tolower(*name));
-	  GWEN_BufferedIO_Write(bio, name+1);
-	  GWEN_BufferedIO_Write(bio, "=");
-	  GWEN_BufferedIO_Write(bio, elemPrefix);
-	  GWEN_BufferedIO_WriteLine(bio, "_List2_new();");
-
-	  GWEN_BufferedIO_WriteLine(bio, "  if (1) {");
-	  GWEN_BufferedIO_WriteLine(bio, "    GWEN_DB_NODE *dbT;");
-	  GWEN_BufferedIO_Write(bio, "    ");
-	  GWEN_BufferedIO_Write(bio, elemType);
-	  GWEN_BufferedIO_WriteLine(bio, " *e;");
-	  GWEN_BufferedIO_WriteLine(bio, "");
-	  GWEN_BufferedIO_Write(bio,
-				"    dbT=GWEN_DB_GetGroup(db, "
-				"GWEN_PATH_FLAGS_NAMEMUSTEXIST, \"");
-	  GWEN_BufferedIO_WriteChar(bio, tolower(*name));
-	  GWEN_BufferedIO_Write(bio, name+1);
-	  GWEN_BufferedIO_WriteLine(bio, "\");");
-	  GWEN_BufferedIO_WriteLine(bio, "    if (dbT) {");
-	  GWEN_BufferedIO_WriteLine(bio, "      GWEN_DB_NODE *dbT2;");
-	  GWEN_BufferedIO_WriteLine(bio, "");
-  
-	  GWEN_BufferedIO_Write(bio,
-				"    dbT2=GWEN_DB_FindFirstGroup(dbT, \"");
-	  GWEN_BufferedIO_Write(bio, "element");
-	  GWEN_BufferedIO_WriteLine(bio, "\");");
-  
-	  /* while (e) */
-	  GWEN_BufferedIO_WriteLine(bio, "      while(dbT2) {");
-  
-	  /* e=ElemType_fromDb(e) */
-	  GWEN_BufferedIO_Write(bio, "        e=");
-	  GWEN_BufferedIO_Write(bio, elemPrefix);
-	  GWEN_BufferedIO_WriteLine(bio, "_fromDb(dbT2);");
-  
-	  /* if (!e) */
-	  GWEN_BufferedIO_WriteLine(bio, "        if (!e) {");
-	  GWEN_BufferedIO_Write(bio, "          "
-                                "DBG_ERROR(0, \"Bad element for type \\\"");
-          GWEN_BufferedIO_Write(bio, elemType);
-          GWEN_BufferedIO_WriteLine(bio, "\\\"\");");
-	  GWEN_BufferedIO_WriteLine(bio, "          "
-				    "if (GWEN_Logger_GetLevel(0)>="
-                                    "GWEN_LoggerLevelDebug)");
-	  GWEN_BufferedIO_WriteLine(bio, "            "
-				    "GWEN_DB_Dump(dbT2, stderr, 2);");
-	  GWEN_BufferedIO_Write(bio, "          ");
-	  GWEN_BufferedIO_Write(bio, prefix);
-	  GWEN_BufferedIO_WriteLine(bio, "_free(st);");
-	  GWEN_BufferedIO_WriteLine(bio, "          return 0;");
-	  GWEN_BufferedIO_WriteLine(bio, "        } /* if !e */");
-  
-	  /* ElemType_List_Add(e, st->NAME); */
-	  GWEN_BufferedIO_Write(bio, "        ");
-	  GWEN_BufferedIO_Write(bio, elemPrefix);
-	  GWEN_BufferedIO_Write(bio, "_List2_PushBack(st->");
-	  GWEN_BufferedIO_WriteChar(bio, tolower(*name));
-	  GWEN_BufferedIO_Write(bio, name+1);
-	  GWEN_BufferedIO_WriteLine(bio, ", e);");
-
-	  GWEN_BufferedIO_Write(bio,"        "
-                                "dbT2=GWEN_DB_FindNextGroup(dbT2, \"");
-	  GWEN_BufferedIO_Write(bio, "element");
-	  GWEN_BufferedIO_WriteLine(bio, "\");");
-  
-	  GWEN_BufferedIO_WriteLine(bio, "      } /* while */");
-  
-	  GWEN_BufferedIO_WriteLine(bio, "    } /* if (dbT) */");
-  
-	  GWEN_BufferedIO_WriteLine(bio, "  } /* if (1) */");
-	}
-	else if (strcasecmp(typ, "GWEN_STRINGLIST")==0) {
-          GWEN_BufferedIO_WriteLine(bio, "  if (1) {");
-          GWEN_BufferedIO_WriteLine(bio, "    int i;");
-          GWEN_BufferedIO_WriteLine(bio, "");
-          GWEN_BufferedIO_WriteLine(bio, "    for (i=0; ; i++) {");
-          GWEN_BufferedIO_WriteLine(bio, "      const char *s;");
-          GWEN_BufferedIO_WriteLine(bio, "");
-          GWEN_BufferedIO_Write(bio, "      s=GWEN_DB_GetCharValue(db, \"");
-          GWEN_BufferedIO_WriteChar(bio, tolower(*name));
-          GWEN_BufferedIO_Write(bio, name+1);
-          GWEN_BufferedIO_WriteLine(bio, "\", i, 0);");
-          GWEN_BufferedIO_WriteLine(bio, "      if (!s)");
-          GWEN_BufferedIO_WriteLine(bio, "        break;");
-          GWEN_BufferedIO_Write(bio, "      ");
-          GWEN_BufferedIO_Write(bio, prefix);
-          GWEN_BufferedIO_Write(bio, "_Add");
-          GWEN_BufferedIO_WriteChar(bio, toupper(*name));
-          GWEN_BufferedIO_Write(bio, name+1);
-          GWEN_BufferedIO_WriteLine(bio, "(st, s, 0);");
-          GWEN_BufferedIO_WriteLine(bio, "    } /* for */");
-          GWEN_BufferedIO_WriteLine(bio, "  }");
-	}
-        else {
-	  isPtr=atoi(get_property(n, "ptr", "0"));
-
-          if (isPtr) {
-	    if (strcasecmp(typ, "char")!=0) {
-	      GWEN_BufferedIO_WriteLine(bio, "  if (1) {");
-	      GWEN_BufferedIO_WriteLine(bio, "    GWEN_DB_NODE *dbT;");
-	      GWEN_BufferedIO_WriteLine(bio, "");
-	      GWEN_BufferedIO_Write(bio,
-				    "    dbT=GWEN_DB_GetGroup(db, "
-				    "GWEN_PATH_FLAGS_NAMEMUSTEXIST, \"");
-	      GWEN_BufferedIO_WriteChar(bio, tolower(*name));
-	      GWEN_BufferedIO_Write(bio, name+1);
-	      GWEN_BufferedIO_WriteLine(bio, "\");");
-	      GWEN_BufferedIO_Write(bio, "    if (dbT)");
+	    prefix=get_struct_property(node, "prefix", 0);
+	    assert(prefix);
+    
+	    /* create list code */
+	    elemType=GWEN_XMLNode_GetProperty(n, "elemType", 0);
+	    if (!elemType) {
+	      DBG_ERROR(0, "No \"type\" for list type \"%s\"", typ);
+	      return -1;
 	    }
+    
+	    elemNode=get_typedef(n, elemType);
+	    if (!elemNode) {
+	      DBG_ERROR(0, "Undefined type %s", elemType);
+	      return -1;
+	    }
+	    elemPrefix=GWEN_XMLNode_GetProperty(elemNode, "prefix", 0);
+	    if (!elemPrefix) {
+	      DBG_ERROR(0, "No \"prefix\" for type \"%s\" (within %s)",
+			elemType, typ);
+	      return -1;
+	    }
+    
+	    /* actually generate the code */
+	    GWEN_BufferedIO_Write(bio, "  st->");
+	    GWEN_BufferedIO_WriteChar(bio, tolower(*name));
+	    GWEN_BufferedIO_Write(bio, name+1);
+	    GWEN_BufferedIO_Write(bio, "=");
+	    GWEN_BufferedIO_Write(bio, elemPrefix);
+	    GWEN_BufferedIO_WriteLine(bio, "_List_new();");
+  
+	    GWEN_BufferedIO_WriteLine(bio, "  if (1) {");
+	    GWEN_BufferedIO_WriteLine(bio, "    GWEN_DB_NODE *dbT;");
+	    GWEN_BufferedIO_Write(bio, "    ");
+	    GWEN_BufferedIO_Write(bio, elemType);
+	    GWEN_BufferedIO_WriteLine(bio, " *e;");
+	    GWEN_BufferedIO_WriteLine(bio, "");
+	    GWEN_BufferedIO_Write(bio,
+				  "    dbT=GWEN_DB_GetGroup(db, "
+				  "GWEN_PATH_FLAGS_NAMEMUSTEXIST, \"");
+	    GWEN_BufferedIO_WriteChar(bio, tolower(*name));
+	    GWEN_BufferedIO_Write(bio, name+1);
+	    GWEN_BufferedIO_WriteLine(bio, "\");");
+	    GWEN_BufferedIO_WriteLine(bio, "    if (dbT) {");
+	    GWEN_BufferedIO_WriteLine(bio, "      GWEN_DB_NODE *dbT2;");
+	    GWEN_BufferedIO_WriteLine(bio, "");
+    
+	    GWEN_BufferedIO_Write(bio,
+				  "      dbT2=GWEN_DB_FindFirstGroup(dbT, \"");
+	    GWEN_BufferedIO_Write(bio, "element");
+	    GWEN_BufferedIO_WriteLine(bio, "\");");
+    
+	    /* while (e) */
+	    GWEN_BufferedIO_WriteLine(bio, "      while(dbT2) {");
+    
+	    /* e=ElemType_fromDb(e) */
+	    GWEN_BufferedIO_Write(bio, "        e=");
+	    GWEN_BufferedIO_Write(bio, elemPrefix);
+	    GWEN_BufferedIO_WriteLine(bio, "_fromDb(dbT2);");
+    
+	    /* if (!e) */
+	    GWEN_BufferedIO_WriteLine(bio, "        if (!e) {");
+	    GWEN_BufferedIO_Write(bio, "          "
+				  "DBG_ERROR(0, \"Bad element for type \\\"");
+	    GWEN_BufferedIO_Write(bio, elemType);
+	    GWEN_BufferedIO_WriteLine(bio, "\\\"\");");
+	    GWEN_BufferedIO_WriteLine(bio, "          "
+				      "if (GWEN_Logger_GetLevel(0)>="
+				      "GWEN_LoggerLevelDebug)");
+	    GWEN_BufferedIO_WriteLine(bio, "            "
+				      "GWEN_DB_Dump(dbT2, stderr, 2);");
+	    GWEN_BufferedIO_Write(bio, "          ");
+	    GWEN_BufferedIO_Write(bio, prefix);
+	    GWEN_BufferedIO_WriteLine(bio, "_free(st);");
+	    GWEN_BufferedIO_WriteLine(bio, "          return 0;");
+	    GWEN_BufferedIO_WriteLine(bio, "        }");
+    
+	    /* ElemType_List_Add(e, st->NAME); */
+	    GWEN_BufferedIO_Write(bio, "        ");
+	    GWEN_BufferedIO_Write(bio, elemPrefix);
+	    GWEN_BufferedIO_Write(bio, "_List_Add(e, st->");
+	    GWEN_BufferedIO_WriteChar(bio, tolower(*name));
+	    GWEN_BufferedIO_Write(bio, name+1);
+	    GWEN_BufferedIO_Write(bio, ");");
+    
+	    GWEN_BufferedIO_Write(bio,
+				  "    dbT2=GWEN_DB_FindNextGroup(dbT2, \"");
+	    GWEN_BufferedIO_Write(bio, "element");
+	    GWEN_BufferedIO_WriteLine(bio, "\");");
+    
+	    GWEN_BufferedIO_WriteLine(bio, "      } /* while */");
+    
+	    GWEN_BufferedIO_WriteLine(bio, "    } /* if (dbT) */");
+    
+	    GWEN_BufferedIO_WriteLine(bio, "  } /* if (1) */");
 	  }
-          GWEN_BufferedIO_Write(bio, "  ");
-          GWEN_BufferedIO_Write(bio, prefix);
-          GWEN_BufferedIO_Write(bio, "_Set");
-          GWEN_BufferedIO_WriteChar(bio, toupper(*name));
-          GWEN_BufferedIO_Write(bio, name+1);
-          GWEN_BufferedIO_Write(bio, "(st, ");
-
-          rv=write_code_fromdbArg_c(args, n, bio);
-          if (rv)
-            return rv;
-          GWEN_BufferedIO_WriteLine(bio, ");");
-	  if (isPtr && strcasecmp(typ, "char")!=0) {
+	  else if (strcasecmp(mode, "list2")==0) {
+	    const char *prefix;
+	    const char *elemType;
+	    const char *elemPrefix;
+	    GWEN_XMLNODE *elemNode;
+  
+	    prefix=get_struct_property(node, "prefix", 0);
+	    assert(prefix);
+    
+	    /* create list code */
+	    elemType=GWEN_XMLNode_GetProperty(n, "elemType", 0);
+	    if (!elemType) {
+	      DBG_ERROR(0, "No \"type\" for list type \"%s\"", typ);
+	      return -1;
+	    }
+    
+	    elemNode=get_typedef(node, elemType);
+	    if (!elemNode) {
+	      DBG_ERROR(0, "Undefined type %s", elemType);
+	      return -1;
+	    }
+	    elemPrefix=GWEN_XMLNode_GetProperty(elemNode, "prefix", 0);
+	    if (!elemPrefix) {
+	      DBG_ERROR(0, "No \"prefix\" for type \"%s\" (within %s)",
+			elemType, typ);
+	      return -1;
+	    }
+    
+	    /* actually generate the code */
+	    GWEN_BufferedIO_Write(bio, "  st->");
+	    GWEN_BufferedIO_WriteChar(bio, tolower(*name));
+	    GWEN_BufferedIO_Write(bio, name+1);
+	    GWEN_BufferedIO_Write(bio, "=");
+	    GWEN_BufferedIO_Write(bio, elemPrefix);
+	    GWEN_BufferedIO_WriteLine(bio, "_List2_new();");
+  
+	    GWEN_BufferedIO_WriteLine(bio, "  if (1) {");
+	    GWEN_BufferedIO_WriteLine(bio, "    GWEN_DB_NODE *dbT;");
+	    GWEN_BufferedIO_Write(bio, "    ");
+	    GWEN_BufferedIO_Write(bio, elemType);
+	    GWEN_BufferedIO_WriteLine(bio, " *e;");
+	    GWEN_BufferedIO_WriteLine(bio, "");
+	    GWEN_BufferedIO_Write(bio,
+				  "    dbT=GWEN_DB_GetGroup(db, "
+				  "GWEN_PATH_FLAGS_NAMEMUSTEXIST, \"");
+	    GWEN_BufferedIO_WriteChar(bio, tolower(*name));
+	    GWEN_BufferedIO_Write(bio, name+1);
+	    GWEN_BufferedIO_WriteLine(bio, "\");");
+	    GWEN_BufferedIO_WriteLine(bio, "    if (dbT) {");
+	    GWEN_BufferedIO_WriteLine(bio, "      GWEN_DB_NODE *dbT2;");
+	    GWEN_BufferedIO_WriteLine(bio, "");
+    
+	    GWEN_BufferedIO_Write(bio,
+				  "    dbT2=GWEN_DB_FindFirstGroup(dbT, \"");
+	    GWEN_BufferedIO_Write(bio, "element");
+	    GWEN_BufferedIO_WriteLine(bio, "\");");
+    
+	    /* while (e) */
+	    GWEN_BufferedIO_WriteLine(bio, "      while(dbT2) {");
+    
+	    /* e=ElemType_fromDb(e) */
+	    GWEN_BufferedIO_Write(bio, "        e=");
+	    GWEN_BufferedIO_Write(bio, elemPrefix);
+	    GWEN_BufferedIO_WriteLine(bio, "_fromDb(dbT2);");
+    
+	    /* if (!e) */
+	    GWEN_BufferedIO_WriteLine(bio, "        if (!e) {");
+	    GWEN_BufferedIO_Write(bio, "          "
+				  "DBG_ERROR(0, \"Bad element for type \\\"");
+	    GWEN_BufferedIO_Write(bio, elemType);
+	    GWEN_BufferedIO_WriteLine(bio, "\\\"\");");
+	    GWEN_BufferedIO_WriteLine(bio, "          "
+				      "if (GWEN_Logger_GetLevel(0)>="
+				      "GWEN_LoggerLevelDebug)");
+	    GWEN_BufferedIO_WriteLine(bio, "            "
+				      "GWEN_DB_Dump(dbT2, stderr, 2);");
+	    GWEN_BufferedIO_Write(bio, "          ");
+	    GWEN_BufferedIO_Write(bio, prefix);
+	    GWEN_BufferedIO_WriteLine(bio, "_free(st);");
+	    GWEN_BufferedIO_WriteLine(bio, "          return 0;");
+	    GWEN_BufferedIO_WriteLine(bio, "        } /* if !e */");
+    
+	    /* ElemType_List_Add(e, st->NAME); */
+	    GWEN_BufferedIO_Write(bio, "        ");
+	    GWEN_BufferedIO_Write(bio, elemPrefix);
+	    GWEN_BufferedIO_Write(bio, "_List2_PushBack(st->");
+	    GWEN_BufferedIO_WriteChar(bio, tolower(*name));
+	    GWEN_BufferedIO_Write(bio, name+1);
+	    GWEN_BufferedIO_WriteLine(bio, ", e);");
+  
+	    GWEN_BufferedIO_Write(bio,"        "
+				  "dbT2=GWEN_DB_FindNextGroup(dbT2, \"");
+	    GWEN_BufferedIO_Write(bio, "element");
+	    GWEN_BufferedIO_WriteLine(bio, "\");");
+    
+	    GWEN_BufferedIO_WriteLine(bio, "      } /* while */");
+    
+	    GWEN_BufferedIO_WriteLine(bio, "    } /* if (dbT) */");
+    
+	    GWEN_BufferedIO_WriteLine(bio, "  } /* if (1) */");
+	  }
+	  else if (strcasecmp(typ, "GWEN_STRINGLIST")==0) {
+	    GWEN_BufferedIO_WriteLine(bio, "  if (1) {");
+	    GWEN_BufferedIO_WriteLine(bio, "    int i;");
+	    GWEN_BufferedIO_WriteLine(bio, "");
+	    GWEN_BufferedIO_WriteLine(bio, "    for (i=0; ; i++) {");
+	    GWEN_BufferedIO_WriteLine(bio, "      const char *s;");
+	    GWEN_BufferedIO_WriteLine(bio, "");
+	    GWEN_BufferedIO_Write(bio, "      s=GWEN_DB_GetCharValue(db, \"");
+	    GWEN_BufferedIO_WriteChar(bio, tolower(*name));
+	    GWEN_BufferedIO_Write(bio, name+1);
+	    GWEN_BufferedIO_WriteLine(bio, "\", i, 0);");
+	    GWEN_BufferedIO_WriteLine(bio, "      if (!s)");
+	    GWEN_BufferedIO_WriteLine(bio, "        break;");
+	    GWEN_BufferedIO_Write(bio, "      ");
+	    GWEN_BufferedIO_Write(bio, prefix);
+	    GWEN_BufferedIO_Write(bio, "_Add");
+	    GWEN_BufferedIO_WriteChar(bio, toupper(*name));
+	    GWEN_BufferedIO_Write(bio, name+1);
+	    GWEN_BufferedIO_WriteLine(bio, "(st, s, 0);");
+	    GWEN_BufferedIO_WriteLine(bio, "    } /* for */");
 	    GWEN_BufferedIO_WriteLine(bio, "  }");
 	  }
-        }
+	  else {
+	    isPtr=atoi(get_property(n, "ptr", "0"));
+  
+	    if (isPtr) {
+	      if (strcasecmp(typ, "char")!=0) {
+		GWEN_BufferedIO_WriteLine(bio, "  if (1) {");
+		GWEN_BufferedIO_WriteLine(bio, "    GWEN_DB_NODE *dbT;");
+		GWEN_BufferedIO_WriteLine(bio, "");
+		GWEN_BufferedIO_Write(bio,
+				      "    dbT=GWEN_DB_GetGroup(db, "
+				      "GWEN_PATH_FLAGS_NAMEMUSTEXIST, \"");
+		GWEN_BufferedIO_WriteChar(bio, tolower(*name));
+		GWEN_BufferedIO_Write(bio, name+1);
+		GWEN_BufferedIO_WriteLine(bio, "\");");
+		GWEN_BufferedIO_Write(bio, "    if (dbT)");
+	      }
+	    }
+	    if (isPtr && strcasecmp(typ, "char")!=0) {
+	      GWEN_BufferedIO_Write(bio, " st->");
+	      GWEN_BufferedIO_Write(bio, name);
+	      GWEN_BufferedIO_Write(bio, "=");
+	      rv=write_code_fromdbArg_c(args, n, bio);
+	      if (rv)
+		return rv;
+	      GWEN_BufferedIO_WriteLine(bio, ";");
+	    }
+	    else {
+	      GWEN_BufferedIO_Write(bio, "  ");
+	      GWEN_BufferedIO_Write(bio, prefix);
+	      GWEN_BufferedIO_Write(bio, "_Set");
+	      GWEN_BufferedIO_WriteChar(bio, toupper(*name));
+	      GWEN_BufferedIO_Write(bio, name+1);
+	      GWEN_BufferedIO_Write(bio, "(st, ");
+  
+	      rv=write_code_fromdbArg_c(args, n, bio);
+	      if (rv)
+		return rv;
+	      GWEN_BufferedIO_WriteLine(bio, ");");
+	    }
+  
+	    if (isPtr && strcasecmp(typ, "char")!=0) {
+	      GWEN_BufferedIO_WriteLine(bio, "  }");
+	    }
+	  }
+	}
       }
     }
     n=GWEN_XMLNode_GetNextTag(n);
